from SpectrumCommon.Const.PC import *
import SpectrumCommon.Const.PC.PCTags as pcmvt
from SpectrumCommon.Modvars.GB.GBDefs import *

pc_program_area_types_dim = 'program_area_types'
pc_program_area_types_ids = {
    IH_MATERNAL_NEWBORN_HEALTH_MST_ID : 'IH_MATERNAL_NEWBORN_HEALTH_MST_ID'   ,
    IH_SEXUAL_REPROD_HEALTH_MST_ID    : 'IH_SEXUAL_REPROD_HEALTH_MST_ID',
    UH_ChildHealthMstID               : 'UH_ChildHealthMstID',
    UH_VaccinationMstID               : 'UH_VaccinationMstID',
    UH_MalariaMstID                   : 'UH_MalariaMstID',
    IH_TB_MST_ID                      : 'IH_TB_MST_ID',
    IH_HIV_AIDS_MST_ID                : 'IH_HIV_AIDS_MST_ID',
    UH_NutritionMstID                 : 'UH_NutritionMstID',
    UH_WASHMstID                      : 'UH_WASHMstID',
    IH_NCD_MST_ID                     : 'IH_NCD_MST_ID',
    UH_MentalNueroSubsUseDisMstID     : 'UH_MentalNueroSubsUseDisMstID',
    UH_AdolescentHealthMstID          : 'UH_AdolescentHealthMstID',
    IH_NTD_MST_ID                     : 'IH_NTD_MST_ID'
}


pc_main_rows_dim = "pc_main_rows_dim"
pc_main_rows_ids = {
        PC_PROG_SPEC_HR_HEADER_ROW_CURR_ID         : 'PROG_SPEC_HR_HEADER_ROW',
        PC_NAT_LEVEL_STAFF_ROW_CURR_ID	  	   : 'NAT_LEVEL_STAFF_ROW',
        PC_REG_LEVEL_STAFF_ROW_CURR_ID   	   : 'REG_LEVEL_STAFF_ROW',
        PC_DIST_LEVEL_STAFF_ROW_CURR_ID	 	   : 'DIST_LEVEL_STAFF_ROW',
        PC_TRAIN_HEADER_ROW_CURR_ID                : 'TRAIN_HEADER_ROW',
        PC_INSERV_REFRESH_TRAIN_ROW_CURR_ID        : 'INSERV_REFRESH_TRAIN_ROW',
        PC_TRAIN_OF_TRAIN_ROW_CURR_ID              : 'TRAIN_OF_TRAIN_ROW',
        PC_DEV_TRAIN_PROG_MAT_ROW_CURR_ID          : 'DEV_TRAIN_PROG_MAT_ROW',
        PC_CHANGE_PRESERV_TRAIN_CURRIC_ROW_CURR_ID : 'CHANGE_PRESERV_TRAIN_CURRIC_ROW',
        PC_SUPP_ACT_ROW_CURR_ID                    : 'SUPP_ACT_ROW',
        PC_SUPERVIS_HEADER_ROW_CURR_ID             : 'SUPERVIS_HEADER_ROW',
        PC_COORD_MEET_ROW_CURR_ID                  : 'COORD_MEET_ROW',
        PC_NAT_STAFF_VISIT_LOCAL_ROW_CURR_ID       : 'NAT_STAFF_VISIT_LOCAL_ROW',
        PC_MONITOR_EVAL_HEADER_ROW_CURR_ID         : 'MONITOR_EVAL_HEADER_ROW',
        PC_DESIGN_ME_FRAME_SYS_ROW_CURR_ID         : 'DESIGN_ME_FRAME_SYS_ROW',
        PC_DESIGN_QC_QA_ROW_CURR_ID                : 'DESIGN_QC_QA_ROW',
        PC_DESIGN_REV_DATA_MGMT_SYS_ROW_CURR_ID    : 'DESIGN_REV_DATA_MGMT_SYS_ROW',
        PC_DATA_COLLECT_ANALYSIS_ROW_CURR_ID       : 'DATA_COLLECT_ANALYSIS_ROW',
        PC_QC_QA_ROW_CURR_ID                       : 'QC_QA_ROW',
        PC_INFRA_EQUIP_HEADER_ROW_CURR_ID          : 'INFRA_EQUIP_HEADER_ROW',
        PC_SIT_ASSESS_INFRA_ROW_CURR_ID            : 'SIT_ASSESS_INFRA_ROW',
        PC_EQUIP_UPGRADES_LOW_TIER_ROW_CURR_ID     : 'EQUIP_UPGRADES_LOW_TIER_ROW',
        PC_EQUIP_UPGRADES_HOSP_ROW_CURR_ID         : 'EQUIP_UPGRADES_HOSP_ROW',
        PC_TRANSPORT_HEADER_ROW_CURR_ID            : 'TRANSPORT_HEADER_ROW',
        PC_SIT_ASSESS_TRANSPORT_ROW_CURR_ID  	   : 'SIT_ASSESS_TRANSPORT_ROW',
        PC_NEW_VEHIC_PURCHASE_ROW_CURR_ID 	   : 'NEW_VEHIC_PURCHASE_ROW',
        PC_VEHIC_OP_MAINT_ROW_CURR_ID       	   : 'VEHIC_OP_MAINT_ROW',
        PC_COMM_MEDIA_HEADER_ROW_CURR_ID           : 'COMM_MEDIA_HEADER_ROW',
        PC_DEV_COMM_STRAT_ROW_CURR_ID              : 'DEV_COMM_STRAT_ROW',
        PC_MASS_MEDIA_COMM_ROW_CURR_ID             : 'MASS_MEDIA_COMM_ROW',
        PC_PRINTED_MATS_ROW_CURR_ID                : 'PRINTED_MATS_ROW',
        PC_SOCIAL_OUTR_ACTIV_ROW_CURR_ID           : 'SOCIAL_OUTR_ACTIV_ROW',
        PC_ADVOC_HEADER_ROW_CURR_ID                : 'ADVOC_HEADER_ROW',
        PC_PLAN_ADVOC_STRAT_ROW_CURR_ID            : 'PLAN_ADVOC_STRAT_ROW',
        PC_ADVOC_ACTS_ROW_CURR_ID                  : 'ADVOC_ACTS_ROW',
        PC_ADVOC_MATS_ROW_CURR_ID                  : 'ADVOC_MATS_ROW',
        PC_GEN_PROG_MGMT_HEADER_ROW_CURR_ID        : 'GEN_PROG_MGMT_HEADER_ROW',
        PC_DESIGN_REV_COUNTRY_STRAT_ROW_CURR_ID    : 'DESIGN_REV_COUNTRY_STRAT_ROW',
        PC_DEV_REVIEW_ANNUAL_WORKPLAN_ROW_CURR_ID  : 'DEV_REVIEW_ANNUAL_WORKPLAN_ROW',
        PC_DEV_REVIEW_HR_PLAN_ROW_CURR_ID	   : 'DEV_REVIEW_HR_PLAN_ROW',
        PC_PROG_COORD_MEET_ROW_CURR_ID             : 'PROG_COORD_MEET_ROW',
        PC_COMMOD_REG_POL_ROW_CURR_ID              : 'COMMOD_REG_POL_ROW',
        PC_SIT_ANALYSIS_ROW_CURR_ID                : 'SIT_ANALYSIS_ROW',
        PC_OFFICE_EQUIP_SUPP_ROW_CURR_ID           : 'OFFICE_EQUIP_SUPP_ROW',
        PC_UTIL_ROW_CURR_ID                        : 'UTIL_ROW',
        PC_OTHER_HEADER_ROW_CURR_ID                : 'OTHER_HEADER_ROW',
        PC_DBL_CLICK_ADD_OTHER_ITEMS_ROW_CURR_ID   : 'DBL_CLICK_ADD_OTHER_ITEMS_ROW'
}
pc_main_hiv_rows_dim = "pc_main_hiv_rows_dim"
pc_main_hiv_rows_ids = {
    
        PC_PROG_SPEC_HR_HEADER_ROW_CURR_ID         : 'PROG_SPEC_HR_HEADER_ROW',
        PC_NAT_LEVEL_STAFF_ROW_CURR_ID	  	   : 'NAT_LEVEL_STAFF_ROW',
        PC_REG_LEVEL_STAFF_ROW_CURR_ID   	   : 'REG_LEVEL_STAFF_ROW',
        PC_DIST_LEVEL_STAFF_ROW_CURR_ID	 	   : 'DIST_LEVEL_STAFF_ROW',
        PC_TRAIN_HEADER_ROW_CURR_ID                : 'TRAIN_HEADER_ROW',
        PC_INSERV_REFRESH_TRAIN_ROW_CURR_ID        : 'INSERV_REFRESH_TRAIN_ROW',
        PC_TRAIN_OF_TRAIN_ROW_CURR_ID              : 'TRAIN_OF_TRAIN_ROW',
        PC_DEV_TRAIN_PROG_MAT_ROW_CURR_ID          : 'DEV_TRAIN_PROG_MAT_ROW',
        PC_CHANGE_PRESERV_TRAIN_CURRIC_ROW_CURR_ID : 'CHANGE_PRESERV_TRAIN_CURRIC_ROW',
        PC_SUPP_ACT_ROW_CURR_ID                    : 'SUPP_ACT_ROW',
        PC_SUPERVIS_HEADER_ROW_CURR_ID             : 'SUPERVIS_HEADER_ROW',
        PC_COORD_MEET_ROW_CURR_ID                  : 'COORD_MEET_ROW',
        PC_NAT_STAFF_VISIT_LOCAL_ROW_CURR_ID       : 'NAT_STAFF_VISIT_LOCAL_ROW',
        PC_MONITOR_EVAL_HEADER_ROW_CURR_ID         : 'MONITOR_EVAL_HEADER_ROW',
        PC_DESIGN_ME_FRAME_SYS_ROW_CURR_ID         : 'DESIGN_ME_FRAME_SYS_ROW',
        PC_DESIGN_QC_QA_ROW_CURR_ID                : 'DESIGN_QC_QA_ROW',
        PC_DESIGN_REV_DATA_MGMT_SYS_ROW_CURR_ID    : 'DESIGN_REV_DATA_MGMT_SYS_ROW',
        PC_DATA_COLLECT_ANALYSIS_ROW_CURR_ID       : 'DATA_COLLECT_ANALYSIS_ROW',
        PC_QC_QA_ROW_CURR_ID                       : 'QC_QA_ROW',
        PC_INFRA_EQUIP_HEADER_ROW_CURR_ID          : 'INFRA_EQUIP_HEADER_ROW',
        PC_SIT_ASSESS_INFRA_ROW_CURR_ID            : 'SIT_ASSESS_INFRA_ROW',
        PC_EQUIP_UPGRADES_LOW_TIER_ROW_CURR_ID     : 'EQUIP_UPGRADES_LOW_TIER_ROW',
        PC_EQUIP_UPGRADES_HOSP_ROW_CURR_ID         : 'EQUIP_UPGRADES_HOSP_ROW',
        PC_TRANSPORT_HEADER_ROW_CURR_ID            : 'TRANSPORT_HEADER_ROW',
        PC_SIT_ASSESS_TRANSPORT_ROW_CURR_ID  	   : 'SIT_ASSESS_TRANSPORT_ROW',
        PC_NEW_VEHIC_PURCHASE_ROW_CURR_ID 	   : 'NEW_VEHIC_PURCHASE_ROW',
        PC_VEHIC_OP_MAINT_ROW_CURR_ID       	   : 'VEHIC_OP_MAINT_ROW',
        PC_COMM_MEDIA_HEADER_ROW_CURR_ID           : 'COMM_MEDIA_HEADER_ROW',
        PC_DEV_COMM_STRAT_ROW_CURR_ID              : 'DEV_COMM_STRAT_ROW',
        PC_MASS_MEDIA_COMM_ROW_CURR_ID             : 'MASS_MEDIA_COMM_ROW',
        PC_PRINTED_MATS_ROW_CURR_ID                : 'PRINTED_MATS_ROW',
        PC_SOCIAL_OUTR_ACTIV_ROW_CURR_ID           : 'SOCIAL_OUTR_ACTIV_ROW',
        PC_ADVOC_HEADER_ROW_CURR_ID                : 'ADVOC_HEADER_ROW',
        PC_PLAN_ADVOC_STRAT_ROW_CURR_ID            : 'PLAN_ADVOC_STRAT_ROW',
        PC_ADVOC_ACTS_ROW_CURR_ID                  : 'ADVOC_ACTS_ROW',
        PC_ADVOC_MATS_ROW_CURR_ID                  : 'ADVOC_MATS_ROW',
        PC_GEN_PROG_MGMT_HEADER_ROW_CURR_ID        : 'GEN_PROG_MGMT_HEADER_ROW',
        PC_DESIGN_REV_COUNTRY_STRAT_ROW_CURR_ID    : 'DESIGN_REV_COUNTRY_STRAT_ROW',
        PC_DEV_REVIEW_ANNUAL_WORKPLAN_ROW_CURR_ID  : 'DEV_REVIEW_ANNUAL_WORKPLAN_ROW',
        PC_DEV_REVIEW_HR_PLAN_ROW_CURR_ID	   : 'DEV_REVIEW_HR_PLAN_ROW',
        PC_PROG_COORD_MEET_ROW_CURR_ID             : 'PROG_COORD_MEET_ROW',
        PC_COMMOD_REG_POL_ROW_CURR_ID              : 'COMMOD_REG_POL_ROW',
        PC_SIT_ANALYSIS_ROW_CURR_ID                : 'SIT_ANALYSIS_ROW',
        PC_OFFICE_EQUIP_SUPP_ROW_CURR_ID           : 'OFFICE_EQUIP_SUPP_ROW',
        PC_UTIL_ROW_CURR_ID                        : 'UTIL_ROW',
        #new rows for HIV costing
        PC_POLICY_BASED_INTERV_ROW_CURR_ID             : 'POLICY_BASED_INTERV_ROW',
        PC_SOC_ECON_SUPP_CHILD_ROW_CURR_ID             : 'SOC_ECON_SUPP_CHILD_ROW',
        PC_PREV_VIOL_WOMEN_ROW_CURR_ID                 : 'PREV_VIOL_WOMEN_ROW',
        PC_DECRIM_KEY_POP_BEHAV_ROW_CURR_ID            : 'DECRIM_KEY_POP_BEHAV_ROW',
        PC_PREV_STIGMA_DISCRIM_ROW_CURR_ID             : 'PREV_STIGMA_DISCRIM_ROW',
        PC_COMM_CIVIL_SOC_LEAD_ROW_CURR_ID             : 'COMM_CIVIL_SOC_LEAD_ROW',
        PC_LEGAL_POLICY_REFORMS_ROW_CURR_ID            : 'LEGAL_POLICY_REFORMS_ROW',
        PC_HIV_OTHER_HEADER_ROW_CURR_ID                : 'HIV_OTHER_HEADER_ROW',
        PC_HIV_DBL_CLICK_ADD_OTHER_ITEMS_ROW_CURR_ID   : 'HIV_DBL_CLICK_ADD_OTHER_ITEMS_ROW'
}

pc_cost_cat_records_dim = 'cost_cat_records_dim'
pc_cost_cat_records_ids = {
        1: 'Human Resources: Programme administration',
        12: 'Human Resources: Partners',
        3: 'Training costs',
        4: 'Meeting costs',
        5: 'External professional services',
        6: 'Communications and promotional materials',
        7: 'Supervision',
        8: 'Transport and vehicles',
        9: 'Indirect and overhead costs',
        10: 'Payment for results',
        11: 'Computer hardware and networking',
        13: 'Other'
}


pc_cost_inputs_records_dim = 'cost_input_records_dim'
pc_cost_inputs_records_ids = {
        1: 'Director',
        2: 'Manager',
        3: 'Officers',
        4: 'Support staff',
        111: 'Technical officer',
        112: 'Programmer',
        113: 'IT computing officer',
        114: 'Statistician',
        131: 'Administrative officer',
        132: 'Clerical officer',
        133: 'Secretary',
        134: 'Accountant',
        135: 'IT computing manager',
        136: 'Cleaner',
        137: 'Public health specialist',
        138: 'Public health officer',
        139: 'Health educator/trainer',
        140: 'Public relations manager',
        141: 'Public relations officer',
        142: 'Transport manager',
        143: 'Transport driver',
        46: 'Training: Participant per diem',
        47: 'Training: Cost of transportation: local',
        48: 'Training: Cost of transportation: regional',
        49: 'Training: Cost of transportation: national',
        50: 'Training: Per diem: national facilitator ',
        51: 'Training: Travel cost: national facilitator',
        52: 'Training: Per diem:  international facilitator ',
        53: 'Training: Travel cost: international facilitator',
        54: 'Training: Hotel stay',
        55: 'Training: Refreshments and lunch   ',
        56: 'Training: Room rental: small',
        57: 'Training: Room rental: medium',
        58: 'Training: Room rental: large',
        59: 'Training: Cost of stationary ',
        60: 'Training: Cost of training material ',
        61: 'Meeting: Participant per diem',
        62: 'Meeting: Cost of transportation: local',
        63: 'Meeting: Cost of transportation: regional',
        64: 'Meeting: Cost of transportation: national',
        65: 'Meeting: Per diem: national facilitator ',
        66: 'Meeting: Travel cost: national facilitator',
        67: 'Meeting: Per diem:  international facilitator ',
        68: 'Meeting: Travel cost: international facilitator',
        69: 'Meeting: Hotel stay',
        70: 'Meeting: Refreshments and lunch   ',
        71: 'Meeting: Room rental: small',
        72: 'Meeting: Room rental: medium',
        73: 'Meeting: Room rental: large',
        74: 'Meeting: Cost of stationary ',
        75: 'Meeting: Cost of meeting material ',
        76: 'Consultants: Domestic transport',
        77: 'Consultants: International transport',
        78: 'Consultant: per diem',
        117: 'Senior consultant: daily rate',
        118: 'Mid-level consultant: daily rate',
        119: 'Junior consultant: daily rate',
        79: 'Fiduciary agent: fees',
        80: 'Fiduciary agent: transport',
        81: 'External audit: fees',
        82: 'External audit: transport',
        83: 'Printing: general',
        84: 'Printing: brochure',
        128: 'Printing: form',
        129: 'Printing: registry',
        130: 'Printing: handbook',
        85: 'Radio spot development',
        86: 'Radio spot airing',
        87: 'TV ad development',
        88: 'TV ad airing',
        89: 'Mobile phone',
        90: 'Air time',
        91: 'Promotional materials: T-shirt',
        92: 'Promotional materials: Pin',
        93: 'Promotional materials: mug',
        155: 'Newspapers (100 word insert)',
        156: 'Wall posters',
        157: 'Flyers / leaflets / guidelines',
        158: 'Opinion poll',
        94: 'Cost per supervisory trip: national staff to regional level',
        95: 'Cost per supervisory trip: national staff to local level',
        115: 'Cost per supervisory trip: regional staff to district level',
        116: 'Cost per supervisory trip: district staff to facility level',
        96: 'Truck',
        97: 'Motorcycle',
        98: 'Bicycle',
        100: 'Sample transport',
        159: 'Car',
        101: 'Office rent',
        102: 'Office electricity',
        103: 'Office: Other utilities',
        104: 'Office: Telephone',
        105: 'Office Internet',
        106: 'Fuel',
        107: 'Office Security',
        108: 'Office Cleaning',
        109: 'Payment for result X',
        110: 'Activity based contract',
        99: 'Laptop computer',
        120: 'Desktop computer',
        121: 'Monitor',
        122: 'Server site (including servers, racks, and secure temperature-controlled facilities)',
        123: 'Site-level local area networks (LANs)',
        124: 'Level 1 peripherals: printers, scanners, Wi-Fi routers, etc.',
        125: 'Level 2 peripherals: copy machines, plotters, LCD projectors, large-screen LCD monitors',
        126: 'Internet connectivity',
        127: 'Cloud hosting',
        161: 'Printer',
        162: 'Photocopier ',
        163: 'IT monitoring device (production)',
        164: 'IT monitoring device (distribution)',
        144: 'Superintendent',
        145: 'Police officer',
        146: 'Lawyer',
        147: 'Legal officer',
        148: 'President / Prime Minister',
        149: 'Minister',
        150: 'Members of Parliament',
        151: 'Committee chair',
        152: 'Cabinet',
        153: 'Excise analyst',
        154: 'Tax enforcement officer',
        160: 'Air monitoring device',
}
pc_multi_key_dim = 'pc_multi_key'
pc_multi_key_ids = {
    1:'areaID',
    2:'costSectionID',
    3:'costSubsecionID',
}
pc_costing_sect_dim = 'pc_costing_sect_dim'
pc_costing_sect_ids = {
# 'TB_1': 'Personnel (administration)',
# 'TB_2': 'Training',
# 'TB_3': 'Supervision',
# 'TB_4': 'Monitoring and evaluation',
# 'TB_5': 'Quality control/Quality assurance',
# 'TB_6': 'Programme-specific transport cost',
# 'TB_7': 'Communication, media and outreach',
# 'TB_8': 'Advocacy',
# 'TB_9': 'General programme management and administration',
# 'TB_10': 'Research and innovation',
# 'TB_11': 'Community and civil society engagement, social participation ',
# 'TB_12': 'Multisectoral engagement',
# 'CS_1': 'Programme-specific human resources',
# 'CS_5': 'Infrastructure',
# 'CS_6': 'Transport',
# 'CS_7': 'Communication, media, and outreach',
# 'CS_9': 'General programme management',
# 'CS_10': 'Community health worker training',
# 'CS_11': 'Other',
# 'HS_1': 'Survey populations and health risks',
# 'HS_2': 'Count births, deaths and causes of death',
# 'HS_3': 'Optimize health service data to ensure equitable, quality services for all ',
# 'HS_4': 'Review progress and performance to make informed decisions',
# 'HS_5': 'Enable data use for policy and action',
# 4: 'General programme management and administration',
# 1: 'Tobacco control',
# 1: 'Reduction of harmful gender social norms',
# 2: 'Promoting gender equality by investing in gender-transformative approaches',
# 3: 'Enabling legal environments',
# 4: 'Reduction of stigma and discrimination',
# 5: 'Socioeconomic support for orphans and vulnerable children'
}
pc_admin_levels_dim = 'pc_admin_levels_dim'
pc_admin_levels_ids = {
        PC_PPLI_NATIONAL_ADMIN_LEVEL_CURR_ID : 'National',
        PC_PPLI_PROVINCE_ADMIN_LEVEL_CURR_ID : 'Province',
        PC_PPLI_DISTRICT_ADMIN_LEVEL_CURR_ID : 'District'
}

pc_num_phases_dim = 'pc_num_phases_dim'
pc_num_phases_ids = {
        PC_NO_ACTION_PHASE_0_PPLI_CURR_ID    : 'No action phase',
        PC_PLANNING_PHASE_0_PPLI_CURR_ID     : 'Planning phase',
        PC_EARLY_IMP_PHASE_25_PPLI_CURR_ID   : 'Early impact phase',
        PC_PARTIAL_IMP_PHASE_50_PPLI_CURR_ID : 'Partial impact phase',
        PC_FULL_IMP_PHASE_100_PPLI_CURR_ID   : 'Full impact phase',
}

pc_num_intensity_dim = 'pc_num_intensity_dim'
pc_num_intensity_ids = {
    PC_PHASE_ROW_PPLI_CURR_ID     : 'Phase row',
    PC_INTENSITY_ROW_PPLI_CURR_ID : 'Intensity row',
}

pc_result_shape = lambda d, **kwargs : gb_shape(d, type='1D dict', 
                                                 dim=[pc_multi_key_dim],
                                                 indices=[pc_multi_key_ids],
                                                 innerShape={
                                                        'areaTypeID' : gb_int_shape('Area type ID'),
                                                        'areaID' :  gb_str_shape('Area ID'),
                                                        'costingSectionID' :  gb_str_shape('Costing section ID'),
                                                        'value' : gb_shape('Value', '1D float',
                                                                 dim= [gb_year_dim], 
                                                                 indices=[gb_year_indices])
                                                        },
                                                        has_default=False,
                                                        **kwargs
                                                )



PC_modvar_shapes = {
        pcmvt.PC_VERSION_TAG               : gb_int_shape('PC version, may be deprecated'),
        pcmvt.PC_DATA_CHANGED_TAG          : gb_bool_shape('Data changed, may be deprecated'), 
        pcmvt.PC_PROJ_VALID_TAG            : gb_bool_shape('Projection valid, may be deprecated'), 
        pcmvt.PC_FILE_NAME_TAG             : gb_str_shape('File name, may be deprecated'),                          
        pcmvt.PC_PROJ_VALID_DATE_TAG       : gb_str_shape('Projection valid date, may be deprecated'),    

        pcmvt.PC_AREA_DATA_TAG             : gb_shape('Program area records', type='1D dict', 
                                                 dim=[pc_program_area_types_dim],
                                                 indices=[pc_program_area_types_ids],
                                                 innerShape={
                                                     'areaTypeMstID'  : gb_int_shape('Master ID of program area type'),
                                                     'areaID'  : gb_shape('Program area ID', type='str'),
                                                     'mainRowCosts'  : gb_shape('Main costs', '2D float',
                                                                    dim=[pc_main_rows_dim, gb_year_dim], 
                                                                    indices=[pc_main_rows_ids,  gb_year_indices]),
                                                 },
                                                 has_default=False
                                             ), 
        
        pcmvt.PC_COST_CAT_RECORDS_TAG      : gb_shape('Cost category records', type='1D dict', 
                                                 dim=[pc_cost_cat_records_dim],
                                                 indices=[pc_cost_cat_records_ids],
                                                 innerShape={
                                                     'costCatID'  : gb_int_shape('ID of cost category'),
                                                     'name'  : gb_str_shape('name of cost category'),
                                                     'custom'  : gb_bool_shape('Custom category flag')
                                                 },
                                                 has_default=False
                                             ), 
        pcmvt.PC_COSTING_SECT_RECORDS_TAG  : gb_shape('Cost sector records', type='1D dict', 
                                                 dim=[pc_multi_key_dim],
                                                 indices=[pc_multi_key_ids],
                                                 innerShape={
                                                     'areaTypeID' : gb_int_shape('area type ID'),
                                                     'areaID' :  gb_str_shape('area ID'),
                                                     'costingSectionID' :  gb_str_shape('costing section ID'),
                                                     'stringConstant':  gb_str_shape('strig constant'),
                                                     'name'  : gb_str_shape('name of cost sector category'),
                                                     'custom'  : gb_bool_shape('Custom category flag'),
                                                     'subsectionRecords'  : gb_shape('Subsection records', type='1D dict',
                                                                                     dim=[pc_costing_sect_dim],
                                                                                     indices=[pc_costing_sect_ids],
                                                                                innerShape={
                                                                                            'costingSubsectionID' : gb_str_shape('costing subsection ID'),
                                                                                            'name'  : gb_str_shape('name of costing subsection'),
                                                                                            'stringConstant':  gb_str_shape('strig constant'),
                                                                                            'custom'  : gb_bool_shape('Custom subsection flag'),
                                                                                            # 'costs': gb_shape('Costs', '1D float',
                                                                                            #                     dim= [gb_year_dim], 
                                                                                            #                     indices=[gb_year_indices])
                                                                                        }

                                                                                     )
                                                 },
                                                 has_default=False
                                             ),  
                                             
                                             
        pcmvt.PC_COST_INPUT_RECORDS_TAG    : gb_shape('Cost input records', type='1D dict', 
                                                 dim=[pc_cost_inputs_records_dim],
                                                 indices=[pc_cost_inputs_records_ids],
                                                 innerShape={
                                                     'costInputID'  : gb_int_shape('ID of cost input'),
                                                     'name'  : gb_str_shape('name of cost category'),
                                                     'stringConstant'  : gb_str_shape('string constant for cost input'),
                                                     'costCatID'  : gb_int_shape('ID of cost category'),
                                                     'custom'  : gb_bool_shape('Custom category flag'),
                                                     'note'  : gb_str_shape('Notes'),
                                                     'source'  : gb_str_shape('Source of data'),
                                                     'unitOfMeasure'  : gb_str_shape('Name of unit of measure'),
                                                     'unitOfMeasureStringConstant'  : gb_str_shape('string constant for unit of measure'),
                                                     'unitCost'  : gb_float_shape('Cost per unit')
                                                     
                                                 },
                                                 has_default=False
                                             ), 
        pcmvt.PC_ACT_RECORDS_TAG           : gb_shape('Act records', type='1D dict', 
                                                 dim=[pc_multi_key_dim],
                                                 indices=[pc_multi_key_ids],
                                                 innerShape={
                                                           'areaTypeID' : gb_int_shape('area type ID'),
                                                           'areaID' :  gb_str_shape('area ID'),
                                                           'costingSectionID' :  gb_str_shape('costing section ID'),
                                                           'activityID' :  gb_str_shape('activity ID'),
                                                           'name'  : gb_str_shape('name of activity'),
                                                           'custom'  : gb_bool_shape('Custom activity flag')
                                                    },
                                                        has_default=False
                                                ),
        pcmvt.PC_ACT_LINE_ITEM_RECORDS_TAG : gb_shape('Act Line item records', type='1D dict', 
                                                 dim=[pc_multi_key_dim],
                                                 indices=[pc_multi_key_ids],
                                                 innerShape={
                                                           'areaTypeID' : gb_int_shape('Area type ID'),
                                                           'areaID' :  gb_str_shape('Area ID'),
                                                           'costingSectionID' :  gb_str_shape('Costing section ID'),
                                                           'costingSubsectionID' :  gb_str_shape('Costing subsection ID'),
                                                           'activityID' :  gb_str_shape('Acitivity ID'),
                                                           'costInputID' :  gb_str_shape('Cost input ID'),
                                                           'frequency': gb_shape('frequency', '1D float',
                                                                                                dim= [gb_year_dim],     
                                                                                                indices=[gb_year_indices]
                                                                                                                                                                                               
                                                                                                ),
                                                        'numUnits': gb_shape('Number of units', '1D float',
                                                                             dim= [pc_admin_levels_dim], 
                                                                             indices=[pc_admin_levels_ids]),
                                                        'duration': gb_shape('Duration', '1D float',
                                                                             dim= [pc_admin_levels_dim], 
                                                                             indices=[pc_admin_levels_ids]),
                                                                                                
                                                    },
                                                 has_default=False
                                                ),

        pcmvt.PC_COSTING_SECT_RECORDS_PPLI_TAG  : gb_shape('Cost sector PPLI records', type='1D dict', 
                                                 dim=[pc_multi_key_dim],
                                                 indices=[pc_multi_key_ids],
                                                 innerShape={
                                                     'areaTypeID' : gb_int_shape('area type ID'),
                                                     'areaID' :  gb_str_shape('area ID'),
                                                     'costingSectionID' :  gb_str_shape('costing section ID'),
                                                     'stringConstant':  gb_str_shape('strig constant'),
                                                     'name'  : gb_str_shape('name of cost sector category'),
                                                     'custom'  : gb_bool_shape('Custom category flag'),
                                                     'subsectionRecords'  : gb_shape('Subsection records', type='1D dict',
                                                                                     dim=[pc_costing_sect_dim],
                                                                                     indices=[pc_costing_sect_ids],
                                                                                innerShape={
                                                                                            'costingSubsectionID' : gb_str_shape('costing subsection ID'),
                                                                                            'name'  : gb_str_shape('name of costing subsection'),
                                                                                            'stringConstant':  gb_str_shape('strig constant'),
                                                                                            'custom'  : gb_bool_shape('Custom subsection flag'),
                                                                                            'raiseTaxesPrice'       : gb_year_shape('raiseTaxesPrice'      ),
                                                                                            'raiseTaxesExciseShare' : gb_year_shape('raiseTaxesExciseShare'),
                                                                                            'raiseTaxesTaxShare'    : gb_year_shape('raiseTaxesTaxShare'   ),
                                                                                            'phaseIntensityValues'  : gb_shape('Phase intensity values', '2D float',
                                                                                            dim=[pc_num_intensity_dim, gb_year_dim],
                                                                                            indices=[pc_num_intensity_ids, gb_year_indices]),

                                                                                            # 'costs': gb_shape('Costs', '1D float',
                                                                                            #                     dim= [gb_year_dim], 
                                                                                            #                     indices=[gb_year_indices])
                                                                                        }
                                                                                     )
                                                 },
                                                 has_default=False
                                             ),  
        pcmvt.PC_ACT_RECORDS_PPLI_TAG           : gb_shape('Act PPLI records', type='1D dict', 
                                                 dim=[pc_multi_key_dim],
                                                 indices=[pc_multi_key_ids],
                                                 innerShape={
                                                           'areaTypeID' : gb_int_shape('area type ID'),
                                                           'areaID' :  gb_str_shape('area ID'),
                                                           'costingSectionID' :  gb_str_shape('costing section ID'),
                                                           'activityID' :  gb_str_shape('activity ID'),
                                                           'name'  : gb_str_shape('name of activity'),
                                                           'custom'  : gb_bool_shape('Custom activity flag')
                                                    },
                                                    has_default=False
                                                ),
        pcmvt.PC_ACT_LINE_ITEM_RECORDS_PPLI_TAG : gb_shape('Act Line item PPLI records', type='1D dict', 
                                                 dim=[pc_multi_key_dim],
                                                 indices=[pc_multi_key_ids],
                                                 innerShape={
                                                           'areaTypeID' : gb_int_shape('Area type ID'),
                                                           'areaID' :  gb_str_shape('Area ID'),
                                                           'costingSectionID' :  gb_str_shape('Costing section ID'),
                                                           'costingSubsectionID' :  gb_str_shape('Costing subsection ID'),
                                                           'activityID' :  gb_str_shape('Acitivity ID'),
                                                           'costInputID' :  gb_str_shape('Cost input ID'),
                                                           'quantityPerAdminRegYear': gb_shape('Quantity per admin reg year', '3D float',
                                                                                                dim= [pc_num_phases_dim, pc_admin_levels_dim, gb_year_dim],     
                                                                                                indices=[pc_num_phases_ids, pc_admin_levels_ids, gb_year_indices]
                                                                                                                                                                                               
                                                                                                ),
                                                        'numUnits': gb_shape('Number of units', '1D float',
                                                                             dim= [pc_admin_levels_dim], 
                                                                             indices=[pc_admin_levels_ids]),
                                                        'duration': gb_shape('Duration', '1D float',
                                                                             dim= [pc_admin_levels_dim], 
                                                                             indices=[pc_admin_levels_ids]),
                                                                                                
                                                    },
                                                 has_default=False
                                                ),

        pcmvt.PC_RES_COSTS_BY_SECT_TAG    : pc_result_shape('PC results costs by section'),
        pcmvt.PC_RES_COSTS_BY_ACT_TAG     : pc_result_shape('PC results costs by activity'),
        pcmvt.PC_RES_COSTS_BY_CAT_TAG     : pc_result_shape('PC results costs by category'),

        pcmvt.PC_RES_COSTS_BY_SECT_PPLI_TAG    : pc_result_shape('PC results costs by section PPLI'),
        pcmvt.PC_RES_COSTS_BY_SUBSECT_PPLI_TAG : pc_result_shape('PC results costs by subsection PPLI'),
        pcmvt.PC_RES_COSTS_BY_ACT_PPLI_TAG     : pc_result_shape('PC results costs by activity PPLI'),
        pcmvt.PC_RES_COSTS_BY_CAT_PPLI_TAG     : pc_result_shape('PC results costs by category PPLI'),

        pcmvt.PC_STD_ROW_INFO_META_TAG : gb_shape('Standard row info meta', type='1D dict',
                                                 dim=[pc_main_rows_dim],
                                                 indices=[pc_main_rows_ids],
                                                 innerShape={
                                                     'mstID' : gb_str_shape('Row ID'),
                                                     'name' : gb_str_shape('Row name'),
                                                     'stringConstant' : gb_str_shape('String constant for row'),
                                                     'isAggregate': gb_bool_shape('Aggregate row')
                                                 },
                                                 has_default=False
                                             ),
        pcmvt.PC_HIV_ROW_INFO_META_TAG : gb_shape('Standard row info meta', type='1D dict',
                                                 dim=[pc_main_hiv_rows_dim],
                                                 indices=[pc_main_hiv_rows_ids],
                                                 innerShape={
                                                     'mstID' : gb_str_shape('Row ID'),
                                                     'name' : gb_str_shape('Row name'),
                                                     'stringConstant' : gb_str_shape('String constant for row'),
                                                     'isAggregate': gb_bool_shape('Aggregate row'),
                                                     'isCost': gb_bool_shape('Cost row'),
                                                     'singleLine': gb_bool_shape('Single line row'),
                                                     'hasData': gb_bool_shape('Has data row')
                                                 },
                                                 has_default=False
                                             ),
        pcmvt.PC_ENTER_COSTING_SECT_COSTS_ALL_YEARS_CS_TAG:  gb_bool_shape('Enter costing section costs for all years in costing section mode'),
    
    
}


from datetime import date

import numpy as np

import SpectrumCommon.Const.GB as gbc
import SpectrumCommon.Const.PJ.PJNTags as pjmvt
import SpectrumCommon.Modvars.GB.GBUtil as gbu

import SpectrumCommon.Const.HW.HWConst as hwc
import SpectrumCommon.Const.HW.HWConstLink as hwcl
import SpectrumCommon.Const.HW.HWTags as hwmvt
import SpectrumCommon.Modvars.HW.HWModVarDefs as hwmvd
import SpectrumCommon.Modvars.HW.HWModVarDefsLink as hwmvdl
import SpectrumCommon.Modvars.HW.HWModVarUtil as hwmvu
import SpectrumCommon.Modvars.HW.HWReadDatabases as hwrd

def init_modvars(countryID, first_year, final_year, extra):
    '''
        Initializes Health Workforce MVs for new IHT projections.

            Parameters:
                countryID (string):  Ex: BEN for Benin.

                first_year (int): First year selected by user.

                final_year (int): Final year selected by user. 

                extra (dict): Any other fields.  
 
            Returns:
                mod_vars_rec (dict): MVs.
    '''

    IHT_costing_on = extra[gbc.GB_IHT_ON] if gbc.GB_IHT_ON in extra else False
    TB_costing_on = extra[gbc.GB_TB_COSTING_ON] if gbc.GB_TB_COSTING_ON in extra else False
    LiST_costing_on = extra[gbc.GB_LIST_COSTING_ON] if gbc.GB_LIST_COSTING_ON in extra else False

    costing_mode_mod_vars = {
        pjmvt.GB_IHTOnTag       : IHT_costing_on,
        pjmvt.GB_TBCostingOnTag : TB_costing_on,
        pjmvt.GB_LCOnTag        : LiST_costing_on,
    }

    costing_mode = gbu.get_costing_mode(costing_mode_mod_vars)

    # Load databases
    
    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        staff_salaries_DB = hwrd.HW_load_staff_salaries_DB(countryID)

    value_bool_false = False
    value_bool_true = True

    value_int = 0
    value_flt = 0.0
    value_str = ''
    value_date = date.today().strftime("%B %d, %Y"),   

    num_years_inputs = final_year - first_year  + 1
    num_years_outputs = final_year - first_year + 1

    staff_type_records = hwmvd.HW_init_staff_type_records(extra[hwcl.HW_IS_FACILITY_RECORDS_TAG_V1], num_years_inputs, costing_mode)
    hwmvd.HW_init_staff_type_target_setting_methods(staff_type_records)
    inserv_train_type_records = hwmvd.HW_init_inserv_train_type_records(num_years_inputs)

    num_staff_types = hwmvu.get_HW_num_staff_types(staff_type_records)

    attrition_type_records = []
    retention_incentive_records = []

    # Meta data
    static_staff_list_meta = hwmvd.HW_init_static_staff_list_meta()

    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        hwrd.HW_set_staff_salaries(staff_type_records, staff_salaries_DB)

    ###################################################################################################################
    # 
    #                                             For testing calculations
    #
    ###################################################################################################################

    # HR_RES_TOTAL_STAFF_TAG_V1, HR_RES_TOTAL_COMPENSATION_TAG_V1
    
    # Target setting method is generally set to Facility staffing standards to start. Set to pop norms, existing plans, and no staff for a 
    # couple of staff for testing.
    # primary_care_doc_rec = hrmvu.get_HR_staff_type_rec_from_curr_ID(staff_type_records, hrc.HR_GEN_PRIM_CARE_DOC_CURR_ID)
    # nurses_rec = hrmvu.get_HR_staff_type_rec_from_curr_ID(staff_type_records, hrc.HR_NURSES_CURR_ID)
    # lab_tech_rec = hrmvu.get_HR_staff_type_rec_from_curr_ID(staff_type_records, hrc.HR_LAB_TECHS_ASSIST_CURR_ID)

    # for sl in acu.GBRange(1, hrc.HR_NUM_STAFFING_LEVELS):
    #     for staff_type_rec in staff_type_records:
    #         hrmvu.set_HR_staff_type_num_in_base_year(staff_type_rec, sl, 100.0 * sl)

    #         # HR_RES_TOTAL_COMPENSATION_TAG_V1. We should already have salaries because there are defaults. These will give us results for benefits.
    #         hrmvu.set_HR_staff_type_annual_real_salary_incr(staff_type_rec, 5)
    #         hrmvu.set_HR_staff_type_benefits(staff_type_rec, 3)
    #         hrmvu.set_HR_staff_type_annual_benefits_adj(staff_type_rec, 2)

    #     hrmvu.set_HR_staff_type_target_setting_method(primary_care_doc_rec, sl, hrc.HR_POP_NORMS_MST_ID)
    #     hrmvu.set_HR_staff_type_target_setting_method(nurses_rec, sl, hrc.HR_EXISTING_PLANS_MST_ID)
    #     hrmvu.set_HR_staff_type_target_setting_method(lab_tech_rec, sl, hrc.HR_NO_STAFF_MST_ID)

    # # HR_RES_STAFF_NEEDING_TO_BE_HIRED_TAG_V1
    # attr_type_rec = hrmvd.HR_create_attrition_type_rec('attrType1', 'burn out')
    # attrition_type_records.append(attr_type_rec)

    # retent_incent_rec = hrmvd.HR_create_retent_incent_rec('retentIncent1', 'work from home option', 1)
    # retention_incentive_records.append(retent_incent_rec)

    # # Transaction-type work
    # for st in acu.GBRange(1, num_staff_types):
    #     staff_type_records[st - 1][hrmvf.HR_STAFF_CURR_ATTRIT_PER_YEAR].append(0)
    #     staff_type_records[st - 1][hrmvf.HR_STAFF_PERC_REC_ATTRIT_REDUC_INCENTS].append(0)
    #     staff_type_records[st - 1][hrmvf.HR_STAFF_ANNUAL_COST_ATTRIT_REDUC_INCENTS].append(0)
    #     staff_type_records[st - 1][hrmvf.HR_STAFF_TARGET_RETENT_INCENT_LEVELS].append(np.full((num_years_full), 0).tolist())
    
    # for staff_type_rec in staff_type_records:
    #     hrmvu.set_HR_staff_type_curr_attr_per_year(staff_type_rec, 1, 20)
    #     hrmvu.set_HR_staff_type_perc_receiv_attrit_reduc_incents(staff_type_rec, 1, 70)
    #     hrmvu.set_HR_retent_incent_reduc_attrit(retent_incent_rec, 1, 15)

    mod_vars_rec = {            
        # Inputs / pre-result / calculated
        hwmvt.HW_VERSION_TAG_V1                                : 1,
        hwmvt.HW_DATA_CHANGED_TAG_V1                           : value_bool_false, 
        hwmvt.HW_PROJ_VALID_TAG_V1                             : value_bool_false, 
        hwmvt.HW_FILE_NAME_TAG_V1                              : value_str,                          
        hwmvt.HW_PROJ_VALID_DATE_TAG_V1                        : value_date,    
        hwmvt.HW_COST_INSERV_TRAIN_BY_PERC_SALARY_TAG_V1       : value_bool_true,
        hwmvt.HW_SHOW_PRESERV_TRAIN_BY_INSTITUTION_TAG_V1      : value_bool_false,  
        hwmvt.HW_STAFF_TYPE_RECORDS_TAG_V1                     : staff_type_records,        
        hwmvt.HW_INSERV_TRAIN_TYPE_RECORDS_TAG_V1              : inserv_train_type_records,  
        hwmvt.HW_ATTRITION_TYPE_RECORDS_TAG_V1                 : attrition_type_records,
        hwmvt.HW_RETENTION_INCENTIVE_RECORDS_TAG_V1            : retention_incentive_records,
        hwmvt.HW_TARG_SCALE_UP_VISITED_TAG_V1                  : np.full((hwc.HW_NUM_STAFFING_LEVELS), False).tolist(),
        hwmvt.HW_TARG_ENROLLMENT_VISITED_TAG_V1                : value_bool_false,                
        hwmvt.HW_TARG_GRADUATION_VISITED_TAG_V1                : value_bool_false,
        hwmvt.HW_TARG_RETENT_INCENT_LEVELS_VISITED_TAG_V1      : value_bool_false,
        # hrmvt.HR_PRESERV_ENROLLMENT_TAG_V1                     : 0,
        # hrmvt.HR_PROJECTED_GRADUATION_TAG_V1                   : 0,                  
        # hrmvt.HR_TOTAL_INSERV_TRAIN_COSTS_BY_TRAIN_TYPE_TAG_V1 : 0,
        # hrmvt.HR_CURR_GRAD_PERC_CURR_ENROLL_TAG_V1             : 0,
        # hrmvt.HR_ATTRITION_RATE_TAG_V1                         : 0,
        # hrmvt.HR_COST_PRESERV_TRAIN_TAG_V1                     : 0, 
        # hrmvt.HR_TOTAL_INSERV_TRAIN_COSTS_BY_STAFF_TAG_V1      : 0,  
        # hrmvt.HR_STAFF_NEEDING_TO_BE_HIRED_TAG_V1              : 0,
        # hrmvt.HR_TOTAL_STAFF_TAG_V1                            : 0,          
        # hrmvt.HR_TOTAL_COMPENSATION_TAG_V1                     : 0,
        hwmvt.HW_PROG_MGMT_DATA_TAG_V1                          : hwmvdl.HW_create_prog_mgmt_data_dict(num_years_inputs),

        # Outputs / results 
        hwmvt.HW_RES_TOTAL_STAFF_TAG_V1                         : hwmvd.HW_init_staff_level_staff_type_year_arr(num_staff_types, num_years_outputs, 0.0),
        hwmvt.HW_RES_TOTAL_COMPENSATION_TAG_V1                  : hwmvd.HW_init_staff_level_staff_type_compens_year_arr(num_staff_types, num_years_outputs, 0.0),
        hwmvt.HW_RES_STAFF_NEEDING_TO_BE_HIRED_TAG_V1           : hwmvd.HW_init_staff_level_staff_type_year_arr(num_staff_types, num_years_outputs, 0.0),
        hwmvt.HW_RES_ATTRITION_RATE_TAG_V1                      : hwmvd.HW_init_staff_type_year_arr(num_staff_types, num_years_outputs, 0.0),
        hwmvt.HW_RES_INSERV_TRAIN_COSTS_BY_STAFF_TAG_V1         : hwmvd.HW_init_staff_level_staff_type_year_arr(num_staff_types, num_years_outputs, 0.0),
        hwmvt.HW_RES_INSERV_TRAIN_COSTS_BY_TRAIN_TYPE_TAG_V1    : hwmvd.HW_init_inserv_train_type_year_arr(num_years_outputs, 0.0),
        hwmvt.HW_RES_PRESERV_TRAIN_COSTS_TAG_V1                 : hwmvd.HW_init_staff_type_year_arr(num_staff_types, num_years_outputs, 0.0),
        hwmvt.HW_RES_STAFF_PER_10K_POP_TAG_V1                   : hwmvd.HW_init_staff_level_staff_type_year_arr(num_staff_types, num_years_outputs, 0.0),

        hwmvt.HW_STATIC_STAFF_LIST_META_TAG_V1                    : static_staff_list_meta,
    }

    #Debugging
    #with open('C:\Proj\IHTSampleFile.JSON', 'w') as f:
    #   json.dump(mod_vars_dict, f)

    return mod_vars_rec

    


# Contains methods for setting up OHT ModVars

from os import environ

from AvenirCommon.Database import GB_get_db_json, GB_file_exists

import SpectrumCommon.Const.GB as gbc

import SpectrumCommon.Const.HW.HWConst as hwc
import SpectrumCommon.Const.HW.HWDatabaseConst as hwdbc
import SpectrumCommon.Modvars.HW.HWModVarUtil as hwmvu

#######################################################################################################
#                                                                                                     #
#                                       Database loading methods                                      #
#                                                                                                     #
#######################################################################################################

def HW_load_staff_salaries_DB(country_ID):
    staff_salaries_DB_file_name = hwc.HW_STAFF_SALARIES_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + hwc.HW_STAFF_SALARIES_DB_CURR_VERSION + '.' + gbc.GB_JSON
    
    if GB_file_exists(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_HW_CONTAINER, staff_salaries_DB_file_name):
        staff_salaries_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_HW_CONTAINER, staff_salaries_DB_file_name)
    else:
        staff_salaries_DB = gbc.GB_NOT_FOUND

    return staff_salaries_DB

#######################################################################################################
#                                                                                                     #
#                                       Database loading methods                                      #
#                                                                                                     #
#######################################################################################################

def HW_set_staff_salaries(staff_type_records, staff_salaries_DB):
    if staff_salaries_DB != gbc.GB_NOT_FOUND:
        edu_code_1_salary = staff_salaries_DB[hwdbc.HW_EDU_CODE_1_SALARY_KEY_SSDB]
        edu_code_2_salary = staff_salaries_DB[hwdbc.HW_EDU_CODE_2_SALARY_KEY_SSDB]
        edu_code_3_salary = staff_salaries_DB[hwdbc.HW_EDU_CODE_3_SALARY_KEY_SSDB]
        edu_code_4_salary = staff_salaries_DB[hwdbc.HW_EDU_CODE_4_SALARY_KEY_SSDB]
        edu_code_5_salary = staff_salaries_DB[hwdbc.HW_EDU_CODE_5_SALARY_KEY_SSDB]

        for staff_type_rec in staff_type_records:
            staff_type_ID = hwmvu.get_HW_staff_type_ID(staff_type_rec)

            if staff_type_ID == str(hwc.HW_GENERAL_MEDICAL_PRACTITIONERS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_4_salary)

            # elif staff_type_ID == str(hwc.HW_OB_GYNS_MST_ID):
            #     hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, 0.0)

            # elif staff_type_ID == str(hwc.HW_PEDIATRICIANS_MST_ID):
            #     hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, 0.0)
                                    
            elif staff_type_ID == str(hwc.HW_SPECIAL_MEDICAL_PRACTITIONERS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_5_salary)
            
            # elif staff_type_ID == str(hwc.HW_CLINIC_OFF_SURG_TECHS_MST_ID):
            #     hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, 0.0)
            
            elif staff_type_ID == str(hwc.HW_NURSING_PROFESSIONALS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_MIDWIFERY_PROFESSIONALS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_MIDWIFERY_ASSOCIATE_PROFESSIONALS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_2_salary)
            
            elif staff_type_ID == str(hwc.HW_NURSING_ASSOCIATE_PROFESSIONALS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_2_salary)
            
            elif staff_type_ID == str(hwc.HW_MED_PATH_LAB_TECHS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_PHARM_TECHS_ASSIST_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_MED_IMAGING_THERA_EQUIP_TECHS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_AMBULANCE_WORKERS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_COMM_HLTH_WORKERS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_1_salary)
            
            # elif staff_type_ID == str(hwc.HW_OTHER_STAFF_TYPE_MST_ID):
            #     hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, 0.0)
            
            elif staff_type_ID == str(hwc.HW_DENTISTS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_4_salary)
            
            elif staff_type_ID == str(hwc.HW_DENTAL_ASSIST_THERAPISTS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_MED_DENTAL_PROSTHETIC_REL_TECHS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_PHARMACISTS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_4_salary)
            
            elif staff_type_ID == str(hwc.HW_PARAMED_PRACTITIONERS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_2_salary)
            
            elif staff_type_ID == str(hwc.HW_ENVIRON_OCCUPAT_HEALTH_HYGIENE_PROF_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_DIETICIANS_NUTRITIONISTS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_AUDIOLOGISTS_SPEECH_THERAPISTS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_4_salary)
            
            elif staff_type_ID == str(hwc.HW_OPTOMETRISTS_OPHTHALMIC_OPTICIANS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_5_salary)
            
            elif staff_type_ID == str(hwc.HW_DISPENSING_OPTICIANS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_4_salary)
            
            elif staff_type_ID == str(hwc.HW_ENVIRON_OCCUPAT_HEALTH_INSPEC_ASSO_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_2_salary)
            
            elif staff_type_ID == str(hwc.HW_TRAD_COMP_MED_PROF_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_TRAD_COMP_MED_ASSOC_PROF_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_2_salary)
            
            elif staff_type_ID == str(hwc.HW_HEALTH_CARE_ASSISTS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_1_salary)
            
            elif staff_type_ID == str(hwc.HW_HOME_BASED_PERS_CARE_WORKERS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_1_salary)
            
            elif staff_type_ID == str(hwc.HW_PERS_CARE_WORKERS_HEALTH_SERV_NOT_CLASS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_1_salary)
            
            elif staff_type_ID == str(hwc.HW_PHYSIOTHERAPISTS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)
            
            elif staff_type_ID == str(hwc.HW_PHYSIOTHERAPY_TECHS_ASSISTS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_2_salary)
            
            elif staff_type_ID == str(hwc.HW_HEALTH_ASSOC_PROF_NOT_CLASS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_2_salary)
            
            elif staff_type_ID == str(hwc.HW_PSYCHOLOGISTS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_5_salary)
            
            elif staff_type_ID == str(hwc.HW_SOCIAL_WORK_COUNSELLING_PROF_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_4_salary)
            
            elif staff_type_ID == str(hwc.HW_SOCIAL_WORK_ASSOC_PROF_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)

            elif staff_type_ID == str(hwc.HW_BIOMED_ENGINEER_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_4_salary)

            elif staff_type_ID == str(hwc.HW_HLTH_SERV_MANAGERS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)

            elif staff_type_ID == str(hwc.HW_AGED_CARE_SERVICE_MANAGERS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_3_salary)

            elif staff_type_ID == str(hwc.HW_MED_SECRETARIES_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_2_salary)

            elif staff_type_ID == str(hwc.HW_MED_REC_HEALTH_INFO_TECHS_MST_ID):
                hwmvu.set_HW_staff_type_annual_salary(staff_type_rec, edu_code_2_salary)









import numpy as np

def get_addresses(groups, i, s, d, lim): #function [i, s, d, lim] = get_addresses(groups, i, s, d, lim)

    #% Initiate any sets not so far covered by s
    if len(s)!=0: #~isempty(s)
       fnames = list(s)#s.keys() #fnames = fieldnames(s);
    else:
       fnames = {} #fnames = {};
    #end
       
    for ig in range(len(groups)):#in groups: #ig = 1:length(groups)
        gp = groups[ig] #gp = groups {ig};
        for ig2 in gp: #for ig2 in gp :#ig2 = 1:length(gp) 
            if not np.isin(ig2,fnames): #if ~ismember(gp{ig2},fnames)
                s[ig2] = [] #s[gp[ig2]] = []; #s.(gp{ig2}) = [];
            #end
        #end
    #end

    if len(groups) == 1: #length(groups) == 1
        gp1 = groups[0] #gp1 = groups{1};
        for ig1 in gp1: #for ig1 in range(len(gp1)): #ig1 = 1:length(gp1)
            #lim = lim+1 #lim = lim+1;
            i[ig1] = np.array([lim])
            #i = dict({ig1: lim}) #i[ig1] = lim #i[gp1[ig1]] = lim #i.(gp1{ig1}) = lim;
            s[ig1] = np.append(s[ig1], lim).astype(int) #s[gp1[ig1]] = [s[gp1[ig1]], lim] #s.(gp1{ig1}) = [s.(gp1{ig1}), lim];
            d[lim] = [ig1] #d[lim] = [gp1[ig1]]
            lim = lim+1 #lim = lim+1; #Changed the position here because indexing in Python is not the same as in Matlab
        #end
    #end

    if len(groups) == 2 :#length(groups) == 2
        gp1 = groups[0]; gp2 = groups[1] #gp1 = groups{1}; gp2 = groups{2} #gp1 = groups{1}; gp2 = groups{2};
    
        for ig1 in gp1: #for ig1 in range(len(gp1)): #ig1 = 1:length(gp1)
            i[ig1] = {}
            for ig2 in range(len(gp2)): #for ig2 in range(len(gp2)): #ig2 = 1:length(gp2)
                #lim = lim+1 #lim = lim+1;
                i[ig1][ig2] = np.array([lim])
                #i = dict({ig1: dict({ig2: lim})}) #i[ig1][ig2] = lim #i[gp1[ig1]][gp2[ig2]] = lim #i.(gp1{ig1}).(gp2{ig2}) = lim;
                s[ig1] = np.append(s[ig1], lim).astype(int) #s[gp1[ig1]] = [s[gp1[ig1]], lim] #s.(gp1{ig1}) = [s.(gp1{ig1}), lim];
                s[ig2] = np.append(s[ig2], lim).astype(int) #s[gp2[ig2]] = [s[gp2[ig2]], lim] #s.(gp2{ig2}) = [s.(gp2{ig2}), lim];
                d[lim] = [ig1, ' ', ig2] #d[lim] = [gp1[ig1], ' ', gp2[ig2]] #d{lim} = [gp1{ig1}, ' ', gp2{ig2}];
                lim = lim+1 #lim = lim+1; #Changed the position here because indexing in Python is not the same as in Matlab
            #end
        #end
    #end

    if len(groups) == 3: #length(groups) == 3
        gp1 = groups[0]; gp2 = groups[1]; gp3 = groups[2]#gp1 = groups{1}; gp2 = groups{2}; gp3 = groups{3};
        for ig1 in range(len(gp1)): #for ig1 in range(len(gp1)): #ig1 = 1:length(gp1)
            i[ig1] = {}
            for ig2 in gp2: #for ig2 in range(len(gp2)): #ig2 = 1:length(gp2)
                i[ig1][ig2] = {}
                for ig3 in gp3: #for ig3 in range(len(gp3)): #ig3 = 1:length(gp3)
                    #lim = lim+1#lim = lim+1;
                    i[ig1][ig2][ig3] = np.array([lim])
                    #i = dict({ig1: dict({ig2: dict({ig3 : lim})})}) #i[ig1][ig2][ig3] = lim #i.(gp1{ig1}).(gp2{ig2}).(gp3{ig3}) = lim;
                    s[ig1] = np.append(s[ig1], lim).astype(int)#s.(gp1{ig1}) = [s.(gp1{ig1}), lim];
                    s[ig2] = np.append(s[ig2], lim).astype(int) #s[gp2[ig2]] = [s[gp2[ig2]], lim] #s.(gp2{ig2}) = [s.(gp2{ig2}), lim];
                    s[ig3] = np.append(s[ig3], lim).astype(int) #s[gp3[ig3]] = [s[gp3[ig3]], lim] #s.(gp3{ig3}) = [s.(gp3{ig3}), lim];
                    d[lim] = [ig1,  ' ', ig2,  ' ', ig3] #d[lim] = [gp1[ig1],  ' ', gp2[ig2],  ' ', gp3[ig3]] #d{lim} = [gp1{ig1},  ' ', gp2{ig2},  ' ', gp3{ig3}];
                    lim = lim+1 #lim = lim+1; #Changed the position here because indexing in Python is not the same as in Matlab
                #end
            #end
        #end
    #end

    if len(groups) == 4 :#length(groups) == 4
        gp1 = groups[0]; gp2 = groups[1]; gp3 = groups[2]; gp4 = groups[3] #gp1 = groups{1}; gp2 = groups{2}; gp3 = groups{3}; gp4 = groups{4};
        for ig1 in gp1: #for ig1 in range(len(gp1)): #ig1 = 1:length(gp1)
            i[ig1] = {}
            for ig2 in gp2: #for ig2 in range(len(gp2)): #for ig2 = 1:length(gp2)
                i[ig1][ig2] = {}
                for ig3 in gp3: #for ig3 in range(len(gp3)): #for ig3 = 1:length(gp3)
                    i[ig1][ig2][ig3] = {}
                    for ig4 in gp4: #for ig4 in range(len(gp4)): #for ig4 = 1:length(gp4)
                        #lim = lim+1 #lim = lim+1;
                        #i4 = dict({ig4:lim}); i3 = dict({ig3 : i4}); i2 = dict({ig2: i3})
                        i[ig1][ig2][ig3][ig4] = np.array([lim])
                        #i = dict({ig1: dict({ig2: dict({ig3 : dict({ig4:lim})})})}) #i[ig1][ig2][ig3][ig4] = lim#i[gp1[ig1]][gp2[ig2]][gp3[ig3]][gp4[ig4]] = lim#i.(gp1{ig1}).(gp2{ig2}).(gp3{ig3}).(gp4{ig4}) = lim;
                        s[ig1] = np.append(s[ig1], lim).astype(int)#s[gp1[ig1]] = [s[gp1[ig1]], lim]#s.(gp1{ig1}) = [s.(gp1{ig1}), lim];
                        s[ig2] = np.append(s[ig2], lim).astype(int) #s[gp2[ig2]] = [s[gp2[ig2]], lim] #s.(gp2{ig2}) = [s.(gp2{ig2}), lim];
                        s[ig3] = np.append(s[ig3], lim).astype(int) #s[gp3[ig3]] = [s[gp3[ig3]], lim] #s.(gp3{ig3}) = [s.(gp3{ig3}), lim];
                        s[ig4] = np.append(s[ig4], lim).astype(int) #s[gp4[ig4]] = [s[gp4[ig4]], lim] #s.(gp4{ig4}) = [s.(gp4{ig4}), lim];
                        d[lim] = [ig1,  ' ', ig2,  ' ', ig3,  ' ', ig4]#d[lim] = [gp1[ig1],  ' ', gp2[ig2],  ' ', gp3[ig3],  ' ', gp4[ig4]]#d{lim} = [gp1{ig1},  ' ', gp2{ig2},  ' ', gp3{ig3},  ' ', gp4{ig4}];
                        lim = lim+1 #lim = lim+1; #Changed the position here because indexing in Python is not the same as in Matlab
                    #end
                #end
            #end
        #end
    #end


    if len(groups) == 5: #length(groups) == 5
        gp1 = groups[0]; gp2 = groups[1]; gp3 = groups[2]; gp4 = groups[3]; gp5 = groups[4] #gp1 = groups{1}; gp2 = groups{2}; gp3 = groups{3}; gp4 = groups{4}; gp5 = groups{5};
        for ig1 in gp1: #for ig1 in range(len(gp1)): #ig1 = 1:length(gp1)
            i[ig1] = {}
            for ig2 in gp2: #for ig2 in range(len(gp2)): #ig2 = 1:length(gp2)
                i[ig1][ig2] = {}
                for ig3 in gp3: #for ig3 in range(len(gp3)): #ig3 = 1:length(gp3)
                    i[ig1][ig2][ig3] = {}
                    for ig4 in gp4: #for ig4 in range(len(gp4)): #ig4 = 1:length(gp4)
                        i[ig1][ig2][ig3][ig4] = {}
                        for ig5 in gp5: #for ig5 in range(len(gp5)): #ig5 = 1:length(gp5)
                            #lim = lim+1 #lim = lim+1;
                            i[ig1][ig2][ig3][ig4] [ig5] = np.array([lim])
                            #i = dict({ig1: dict({ig2: dict({ig3 : dict({ig4: dict({ig5: lim})})})})}) #i[ig1][ig2][ig3][ig4][ig5] = lim #i[gp1[ig1]][gp2[ig2]][gp3[ig3]][gp4[ig4]][gp5[ig5]] = lim #i.(gp1{ig1}).(gp2{ig2}).(gp3{ig3}).(gp4{ig4}).(gp5{ig5}) = lim;
                            s[ig1] = np.append(s[ig1], lim).astype(int) #s[gp1[ig1]] = [s[gp1[ig1]], lim] #s.(gp1{ig1}) = [s.(gp1{ig1}), lim];
                            s[ig2] = np.append(s[ig2], lim).astype(int)  #s[gp2[ig2]] = [s[gp2[ig2]], lim]  #s.(gp2{ig2}) = [s.(gp2{ig2}), lim];
                            s[ig3] = np.append(s[ig3], lim).astype(int) #s[gp3[ig3]] = [s[gp3[ig3]], lim] #s.(gp3{ig3}) = [s.(gp3{ig3}), lim];
                            s[ig4] = np.append(s[ig4], lim).astype(int) #s[gp4[ig4]] = [s[gp4[ig4]], lim] #s.(gp4{ig4}) = [s.(gp4{ig4}), lim];
                            s[ig5] = np.append(s[ig5], lim).astype(int) #s[gp5[ig5]] = [s[gp5[ig5]], lim] #s.(gp5{ig5}) = [s.(gp5{ig5}), lim];
                            d[lim] = [ig1,  ' ', ig2,  ' ', ig3,  ' ', ig4,  ' ', ig5] #d[lim] = [gp1[ig1],  ' ', gp2[ig2],  ' ', gp3[ig3],  ' ', gp4[ig4],  ' ', gp5[ig5]] #d{lim} = [gp1{ig1},  ' ', gp2{ig2},  ' ', gp3{ig3},  ' ', gp4{ig4},  ' ', gp5{ig5}];
                            lim = lim+1 #lim = lim+1; #Changed the position here because indexing in Python is not the same as in Matlab
                        #end
                    #end
                #end
            #end
        #end
    #end
    i['nstates'] = lim #i.nstates = lim;

    return([i, s, d, lim])


# Input ModVars

# IS_CALC_TOTAL_MED_EQUIP_COSTS_V1:
# 
#   TB app, Infrastructure and equipment tab, Manage medical equipment per facility radio group. 
#   Enter total cost directly (false), Enter details used to calculate total costs (true) 
#
# IS_CALC_MAINT_COSTS_AS_PERC_MED_EQUIP_COSTS_TAG:
#
#  TB app, Infrastructure and equipment tab
#  Enter as absolute number (false), Calculate percentage of equipment costs (true)

IS_VERSION_TAG_V1                                = '<IS_Version_V1>'
IS_DATA_CHANGED_TAG_V1                           = '<IS_DataChanged_V1>'
IS_FILE_NAME_TAG_V1                              = '<IS_FileName_V1>'    
IS_PROJ_VALID_TAG_V1                             = '<IS_ProjectionValid_V1>'
IS_FACILITY_TYPE_RECORDS_TAG                     = '<IS_FacilityRecords_V2>'   
IS_FACILITY_MED_EQUIP_RECORDS_TAG                = '<IS_FacilityMedEquipRecords_V3>'    
IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG          = '<IS_FacilityMedEquipGroupRecords_V3>' 
IS_FACILITY_FURNITURE_RECORDS_TAG_V1             = '<IS_FacilityFurnitureRecords_V1>'       
IS_PROJ_VALID_DATE_TAG_V1                        = '<IS_ProjectionValidDate_V1>'   
IS_ENTER_CONSTRUCT_COSTS_BY_COMP_TAG_V1          = '<IS_EnterConstructCostsByComponent_V1>'
IS_ENTER_OP_COST_PERC_CONSTRUCT_COST_TAG_V1      = '<IS_EnterOpCostAsPercOfConstructCost_V1>'
IS_FACIL_TARGET_SETTING_METHOD_TAG_V1            = '<IS_FacilityTargetSettingMethod_V1>'        
IS_VEHICLE_TYPE_RECORDS_TAG_V1                   = '<IS_VehicleRecords_V1>'
IS_DIRECT_ENTRY_FUEL_EXPEND_TAG_V1               = '<IS_EnterFuelExpendituresDirectly_V1>'
IS_COST_FUEL_PER_LITER_TAG_V1                    = '<IS_CostOfFuelPerLiter_V1>'
IS_ICT_EQUIPMENT_RECORDS_TAG_V1                  = '<IS_ICT_EquipmentRecords_V1>'
IS_PROG_MGMT_DATA_TAG_V1                         = '<IS_ProgMgmtData_V1>'       
IS_CALC_TOTAL_MED_EQUIP_COSTS_V1                 = '<IS_CalcTotalMedEquipCosts_V1>'       
IS_CALC_MAINT_COSTS_AS_PERC_MED_EQUIP_COSTS_TAG  = '<IS_CalcMaintCostsAsPercMedEquipCosts_V1>'

# Output / Result ModVars

IS_RES_TOTAL_NUM_FAC_AVAIL_TAG                     = '<IS_ResTotalNumFacilitiesAvail_V1>'
IS_RES_TOTAL_COST_BUILD_EQUIP_NEW_FAC_TAG          = '<IS_ResTotalCostBuildEquipNewFacilities_V1>'
IS_RES_NEW_FAC_CONSTRUCT_TAG                       = '<IS_ResNewFacilityConstruction_V1>'
IS_RES_TOTAL_NUM_BEDS_AVAIL_TAG                    = '<IS_ResTotalNumBedsAvail_V1>'
IS_RES_VEHIC_AVAIL_TAG                             = '<IS_ResVehiclesAvail_V1>'
IS_RES_VEHIC_PURCH_TAG                             = '<IS_ResVehiclesPurch_V1>'
IS_RES_VEHIC_PURCH_CAP_COSTS_TAG                   = '<IS_ResVehiclesPurchCapCosts_V1>'
IS_RES_VEHIC_PURCH_CAP_COSTS_BY_FAC_TAG            = '<IS_ResVehiclesPurchCapCostsByFacility_V1>' # For BG
IS_RES_VEHIC_PURCH_CAP_COSTS_OTHER_NON_FAC_TAG     = '<IS_ResVehiclesPurchCapCostsOtherNonFacility_V1>' # For BG
IS_RES_VEHIC_MAINT_REPAIR_COSTS_TAG                = '<IS_ResVehicleMaintRepairCosts_V1>'
IS_RES_VEHIC_MAINT_REPAIR_COSTS_BY_FAC_TAG         = '<IS_ResVehicleMaintRepairCostsByFacility_V1>' # For BG
IS_RES_VEHIC_MAINT_REPAIR_COSTS_OTHER_NON_FAC_TAG  = '<IS_ResVehicleMaintRepairCostsOtherNonFacility_V1>' # For BG
IS_RES_VEHIC_FUEL_COSTS_TAG                        = '<IS_ResVehicleFuelCosts_V1>'
IS_RES_VEHIC_FUEL_COSTS_BY_FAC_TAG                 = '<IS_ResVehicleFuelCostsByFacility_V1>' # For BG
IS_RES_VEHIC_FUEL_COSTS_OTHER_NON_FAC_TAG          = '<IS_ResVehicleFuelCostsOtherNonFacility_V1>' # For BG
IS_RES_VEHIC_DRIVER_SALARIES_TAG                   = '<IS_ResVehicleDriverSalaries_V1>'
IS_RES_VEHIC_DRIVER_SALARIES_BY_FAC_TAG            = '<IS_ResVehicleDriverSalariesByFacility_V1>' # For BG
IS_RES_VEHIC_DRIVER_SALARIES_OTHER_NON_FAC_TAG     = '<IS_ResVehicleDriverSalariesOtherNonFacility_V1>' # For BG
IS_RES_NEW_FAC_FURNITURE_COSTS_TAG                 = '<IS_ResNewFacilityFurnitureCosts_V1>'
IS_RES_FAC_LAND_COSTS_TAG                          = '<IS_ResFacilityLandCosts_V1>'
IS_RES_NEW_FAC_CONSTRUCT_COSTS_TAG                 = '<IS_ResNewFacilityConstructCosts_V1>'
IS_RES_NEW_FAC_MED_EQUIP_COSTS_TAG                 = '<IS_ResNewFacilityMedEquipCosts_V1>'
IS_RES_FAC_REHAB_COSTS_TAG                         = '<IS_ResFacilityRehabCosts_V1>'
IS_RES_FAC_TOTAL_OP_COSTS_TAG                      = '<IS_ResFacilityTotalOpCosts_V1>'
IS_RES_OTHER_FAC_OP_COSTS_TAG                      = '<IS_ResOtherFacilityOpCosts_V1>'
IS_RES_FAC_WATER_COSTS_TAG                         = '<IS_ResFacilityWaterCosts_V1>'
IS_RES_FAC_ELECTRIC_COSTS_TAG                      = '<IS_ResFacilityElectricCosts_V1>'
IS_RES_ICT_AVAIL_TAG                               = '<IS_ResICTAvail_V1>'
IS_RES_ICT_PURCH_TAG                               = '<IS_ResICTPurch_V1>'
IS_RES_ICT_CAP_COSTS_TAG                           = '<IS_ResICTCapCosts_V1>'
IS_RES_ICT_CAP_COSTS_BY_FAC_TAG                    = '<IS_ResICTCapCostsByFacility_V1>' # For BG
IS_RES_ICT_CAP_COSTS_OTHER_NON_FAC_TAG             = '<IS_ResICTCapCostsOtherNonFacility_V1>' # For BG
IS_RES_ICT_MAINT_REPAIR_OP_COSTS_TAG               = '<IS_ResICTMaintRepairOpCosts_V1>'
IS_RES_ICT_MAINT_REPAIR_OP_COSTS_BY_FAC_TAG        = '<IS_ResICTMaintRepairOpCostsByFacility_V1>' # For BG
IS_RES_ICT_MAINT_REPAIR_OP_COSTS_OTHER_NON_FAC_TAG = '<IS_ResICTMaintRepairOpCostsOtherNonFacility_V1>' # For BG
IS_RES_MED_EQUIP_MAINT_CALIB_COSTS_TB_TAG          = '<IS_ResMedEquipMaintCalibCostsTB_V1>'

# Disease-specific costing (DSC), TB costing mode

# "Equipment costs"; Replaces <IS_ResNewFacilityConstructCosts_V1> for TB app because we don't have access 
# to the same inputs without having Infrastructure on and have to therefore calculate it differently.
IS_RES_EQUIPMENT_COSTS_DSC_V1                     = '<IS_ResEquipmentCostsDSC_V1>'

# Meta modvars; the values inside these do not change. 

IS_DEFAULT_MED_EQUIP_GROUP_LIST_META_TAG = '<IS_DefaultMedEquipGroupList_Meta_V1>'


import numpy as np

from SpectrumCommon.Const.TB import *


def TB_HHPropPrvntPresumpDefaults(year_len):
    HHPropPrvntPresump = np.zeros((TB_HHAgeLen, year_len))  
    HHPropPrvntPresump[TB_HH0_4]  += 0.5
    HHPropPrvntPresump[TB_HH5_14] += 0.5
    HHPropPrvntPresump[TB_HH15p]  += 0.5
    return HHPropPrvntPresump

def TB_HHPropPrvntTested(year_len):
    HHPropPrvntTested = np.zeros((TB_HHAgeLen, year_len))
    # HHPropPrvntTested[TB_HH15p] += 1.0
    return HHPropPrvntTested


from SpectrumCommon.Const.GB.GBConst import *

from SpectrumCommon.Modvars.PC.PCModVarDefs import *

def IS_create_prog_mgmt_data_dict(num_years = int):
    return PC_create_prog_mgmt_data_dict(GB_IS, num_years)


# Fields used in HW data structures

HW_F_STAFF_TYPE_ID    = 'staffTypeID' 
HW_F_ATTR_TYPE_ID     = 'attrTypeID'
HW_F_RETENT_INCENT_ID = 'retentIncentID'

# staff records 

HW_STAFF_ID                               = HW_F_STAFF_TYPE_ID               # string;  numeric mstID for defaults. GUID for customs.
HW_STAFF_STR_CONSTANT                     = 'stringConstant'                 # string;  Defaults only.
HW_STAFF_NAME                             = 'name'                           # string;  Holds custom name or overridden default name.
HW_STAFF_ACTIVE                           = 'active'                         # boolean; Is user using this item or not?
HW_STAFF_DIVISION_MST_ID                  = 'staffDivisionMstID'             # integer;
HW_STAFF_CUSTOM                           = 'custom'                         # boolean; user-entered = true; default = false
HW_STAFF_STATIC_MAP_MST_ID                = 'defStaffTypeMapMstID'           # string;  mstID of staff type user maps custom staff types to
# baseline - staff baseline
HW_STAFF_BASE_YEAR                        = 'numInBaseYear'                  # 1d list of integer; by staff level
# baseline - salaries and benefits
HW_STAFF_ANNUAL_SALARY                    = 'annualSalary'                   # float
HW_STAFF_ANNUAL_REAL_SALARY_INCR          = 'annualRealSalaryIncrease'       # float
HW_STAFF_ANNUAL_BENEFITS                  = 'annualBenefits'                 # float
HW_STAFF_ANNUAL_BENEFITS_ADJUST           = 'annualBenefitsAdjustment'       # integer
# baseline - staff time utilization
HW_STAFF_PERC_TIME_SPENT_NOT_TRAIN_SERV   = 'percTimeSpentNotTrainServ'      # integer
HW_STAFF_AVG_DAYS_WORKED_PER_YEAR         = 'avgDaysWorkedPerYear'           # integer
HW_STAFF_HOURS_IN_WORK_DAY                = 'hoursInWorkDay'                 # integer
# baseline - attrition
HW_STAFF_CURR_ATTRIT_PER_YEAR             = 'currAttritPerYear'              # 1d list of integer; by attrition type
# baseline - retention incentives 
HW_STAFF_PERC_REC_ATTRIT_REDUC_INCENTS    = 'percReceivAttritReducIncents'   # 1d list of integer; by retention incentive
HW_STAFF_ANNUAL_COST_ATTRIT_REDUC_INCENTS = 'annCostAttrReducIncents'        # 1d list of double; by retention incentive
# baseline - pre-service training (when not disaggregated by training instution)
HW_STAFF_PRESERV_NUM_SCHOOLS              = 'preservNumSchools'              # integer
HW_STAFF_PRESERV_YEARS_TRAINING           = 'preservYearsTraining'           # integer
HW_STAFF_PRESERV_ANNUAL_COST_PER_STUDENT  = 'preservAnnCostPerStudent'       # float
HW_STAFF_PRESERV_CURR_ENROLLMENT          = 'preservCurrEnrollment'          # integer
HW_STAFF_PRESERV_GRADS_MOST_RECENT_YEAR   = 'gradsMostRecentYear'            # integer
# baseline - pre-service training (when disaggregated by training instution)
HW_STAFF_PRESERV_INST_NAMES                          = 'trainingInstNames'                 # 1d list of string; by training institution 
HW_STAFF_PRESERV_AVG_YEARS_TRAINING_BY_INST          = 'avgYearsTrainingByInst'             # 1d list of integer; by training institution
HW_STAFF_PRESERV_ANNUAL_AVG_COST_PER_STUDENT_BY_INST = 'annualAvgCostPerStudentByInst'      # 1d list of double; by training institution
HW_STAFF_PRESERV_CURR_INTAKE_BY_INST                 = 'currIntakeFirstYearStudentsByInst'  # 1d list of integer; by training institution
HW_STAFF_PRESERV_GRADS_MOST_RECENT_YEAR_BY_INST      = 'gradsMostRecentYearByInst'          # 1d list of integer; by training institution
# baseline - recruitment baseline
HW_STAFF_TOTAL_ANNUAL_RECRUITMENT                    = 'totalAnnualRecruitment'         # integer
HW_STAFF_PERC_NEW_GRADS_PUBLIC_SECTOR                = 'percNewGradsPublicSector'       # float
# baseline - in-service training
HW_STAFF_INSERV_TRAIN_COST_PERC_SALARY               = 'inservTrainCostAsPercSalary'    # float
# target setting - configuration
HW_STAFF_TARGET_SETTING_METHOD                       = 'targetSettingMethod'            # 1d list of integer; by staffing level 
# target setting - target entry and scale-up
HW_STAFF_TARGET_STAFF_POP_RATIO                      = 'targetStaffPopRatio'            # 1d list of float; by staff level
HW_STAFF_TARGET_STAFF_PER_FACILITY                   = 'targetNumPerFacility'           # 2d list of integer; staff level x facility type
HW_STAFF_TARGET_SCALE_UP_EXISTING_PLANS              = 'targetScaleUpExistingPlans'     # 2d list of integer; staff level x year
# target setting - target enrollment
HW_STAFF_TARGET_ENROLLMENT                           = 'targetEnrollment'               # 1d list of integer; by year
# target setting - target graduation
HW_STAFF_TARGET_GRADUATION                           = 'targetGraduation'               # 1d list of integer; by year
# target setting - retention incentives
HW_STAFF_TARGET_RETENT_INCENT_LEVELS                 = 'targetRetentIncentLevels'       # 2d list of integer; retention incentive x year 
# baseNumStaffTypeCountryPrivSect     # integer
# totAnnIntake                        # integer
# attrToReachStaffTargYrAchieved      # integer
# recrToReachStaffTargYrAchieved      # integer
# hirNewGradsTargYrAchieved           # integer
# hirExperStaffTargYrAchieved         # integer
# generalIncrTrainCapacTargYrAchieved # integer
# newGradsPubSectTargYrAchieved       # integer
# dropOutRateByInst                   # 1d list of float; by training institution
# incrTrainCapacInstTargYrAchieved    # 1d list of integer; by training institution
# incrTrainCapacInstTarg              # 1d list of integer; by training institution
# totAnnIntakeByInst                  # 1d list of integer; by training institution
# targTrainCapacity                   # 1d list of integer; by year
# targPubSectRecruit                  # 1d list of integer; by year
# attrToReachStaffTarg                # 1d list of integer; by year
# recrToReachStaffTarg                # 1d list of integer; by year
# hirNewGradsTarg                     # 1d list of integer; by year
# hirExperStaffTarg                   # 1d list of integer; by year
# generalIncrTrainCapacTarg           # 1d list of integer; by year
# newGradsPubSectTarg                 # 1d list of float; by year
# baseStaffReceivGradIncents          # 1d list of integer; by graduation incentive
# annCostMatricIncents                # 1d list of float; by graduation incentive
# baseStaffReceivTransfIncents        # 1d list of integer; by recruitment incentive
# costHirExpStaffIncents              # 1d list of float; by recruitment incentive
# measTargRedAttrPolData              # 2d list of integer; policy scenario x retention incentive
# measIncMatPubSectPolData            # 2d list of integer; policy scenario x graduation incentive
# measIncHirExpStaffPolData           # 2d list of integer; policy scenario x recruitment incentive
# attrReducIncentTarg                 # 2d list of integer; retention incentive x year
# attrReducIncentTargYrAchieved       # 1d list of integer; by retention incentive


# in-service training type records (new)

HW_INSERV_TRAIN_STAFF_TYPE_TO_TRAIN    = 'staffTypeToBeTrainedID' # string; numermic mstID for defaults (still cast as string). GUID for custom interventions.
HW_INSERV_TRAIN_NAME                   = 'name'                   # string
HW_INSERV_TRAIN_COST_PER_STAFF_TRAINED = 'costPerStaffTrained'    # float
HW_INSERV_TRAIN_NUM_STAFF_TO_TRAIN     = 'numStaffToBeTrained'    # 1d list of integer; by year

# attrition type records

HW_ATTRIT_TYPE_ID   = HW_F_ATTR_TYPE_ID   # string; GUID
HW_ATTRIT_TYPE_NAME = 'name'              # string

# retention incentive records

HW_RETENT_INCENT_ID                = HW_F_RETENT_INCENT_ID      # string; GUID
HW_RETENT_INCENT_NAME              = 'name'                     # string
HW_RETENT_INCENT_PERC_REDUC_ATTRIT = 'percReductionAttrition'   # 1d list of integer; by attrition type



import numpy as np

from SpectrumCommon.Const.GB import GB_BaseYear 
from SpectrumCommon.Const.FS import *


def create_absolute_dict(numbers, percents, opt):
    return {
        'absolute': numbers,
        'percents': percents,
        'option': opt
    }

#@autoreload
def init_modvars(countryID, first_year, final_year,  extra):
    #Variables needed for multiple modvars
    #final_year = 2030
    
    scenario_names = ['Scenario 1', 'Scenario 2', 'Scenario 3']


    base_year = GB_BaseYear
    first_year_idx = first_year - base_year
    final_year_idx = final_year - base_year 

    year_zeroes = np.zeros((final_year_idx+1)).tolist() 
    blank_num_perc = create_absolute_dict(year_zeroes.copy(), year_zeroes.copy(), 'absolute')
    
    #modvar creation list
    return {
        # General
        FS_ScenarioNameTag     : scenario_names,

        # MacroEconomics
        ## GDP tab
        FS_MacroGDPTag          : [blank_num_perc for s in scenario_names],

        ## General government expenditure tab
        FS_MacroExpenditureTag  : [blank_num_perc for s in scenario_names],

        ## General government revenue
        FS_MacroTaxesTag        : [blank_num_perc for s in scenario_names],
        FS_MacroNonTaxesTag     : [blank_num_perc for s in scenario_names],
        FS_MacroGrantsTag       : [blank_num_perc for s in scenario_names],
        FS_MacroOtherRevenueTag : [blank_num_perc for s in scenario_names],


        # Health Expenditures
        ## General government health expenditure
        FS_GGHEApproachTag: FS_MANUAL_APPROACH,

        FS_GGHEDomesticTag:[blank_num_perc for s in scenario_names], 
        FS_GGHEExternalTag:[blank_num_perc for s in scenario_names], 

        FS_HealthExpCoeffTag : [{'GDP':0, 'GGHE':0} for s in scenario_names] ,
        FS_PercOfGGETag : [{'baseline':0, 'target':0, 'yearsOfDelay':0, 'yearsUntilArrival':0} for s in scenario_names] ,
        FS_ConstantPricesTag : [{'baseline':0, 'target':0, 'yearsOfDelay':0, 'yearsUntilArrival':0} for s in scenario_names] ,

        ## Private health expenditure
        FS_OutOfPocketTag : [blank_num_perc for s in scenario_names],
        FS_PrePaidRiskPlansTag : [blank_num_perc for s in scenario_names],

        FS_GrowthOptionTag: [False for s in scenario_names] ,
        FS_PrivateHealthCoeffTag : [{'GDP':0, 'GGHE':0} for s in scenario_names] ,

        ## External resources for health
        FS_HealthExpOtherTag : [year_zeroes.copy() for s in scenario_names],
        
        # # Results
        # FS_PlannedExpVSFiscalSpaceTag: [],
        # FS_PlannedExpVSFinancialSpaceTag: [],
        # FS_HealthExpenditureRatioTag: [],
        # FS_PerCapitaTag: [{'GDP':year_zeroes, 'THE':year_zeroes, 'GGHE':year_zeroes} for s in scenario_names],
        
        # ## Macroeconomic results
        # FS_FinancialSpaceTag: [],

        # ## Macroeconomic results
        # FS_MacroPopResTag:[],
        # FS_MacroGDPResTag:[],
        # FS_MacroExpendResTag:[],
        # FS_MacroExpendCompResTag:[],
        # FS_MacroRevenueResTag:[],
        # FS_MacroCurrentPricesTag:[],

        # ## Health expenditure results
        # ## Custom graph results

        
    }


HVFirstYearTag = "<HV_FirstYear_V1>"
HVFinalYearTag = "<HV_FinalYear_V1>"
HVFinalIndexTag = "<HV_FinalIndex_V1>"
HVDataChangedTag = "<HV_DataChanged_V1>"
HVProjectionValidTag = "<HV_ProjectionValid_V1>"
HVProjectionValidDateTag = "<HV_ProjectionValidDate_V1>"
HVFileNameTag = "<HV_FileName_V1>"
HVBehaviorTag = "<HV_Behavior_V1>"
HVAgeFirstSexTag = "<HV_AgeFirstSex_V1>"
HVForceInfTag = "<HV_ForceInf_V1>"
HVInfectiousnessTag = "<HV_Infectiousness_V1>"
HVPrevalenceTag = "<HV_Prevalence_V1>"
HVCondomPercentTag = "<HV_CondomPercent_V1>"
HVTransMultSTITag = "<HV_TransMultSTI_V1>"
HVTransHIVFTag = "<HV_TransHIVF_V1>"
HVTransMultMTag = "<HV_TransMultM_V1>"
HVCondomEffTag = "<HV_CondomEff_V1>"
HVNumPartTag = "<HV_NumPart_V1>"
HVSexActsTag = "<HV_SexActs_V1>"
HVSTIPrevTag = "<HV_STIPrev_V1>"
HVEpidemicStYrTag = "<HV_EpidemicStYr_V1>"
HVIncRecruitmentTag = "<HV_IncRecruitment_V1>"
HVAllRiskOutputTag = "<HV_AllRiskOutput_V1>"
HVMaxAllRiskTag = "<HV_MaxAllRisk_V1>"
HVPercMarriedTag = "<HV_PercMarried_V1>"
HVPerIDUsharingTag = "<HV_PerIDUsharing_V1>"
HVInitialPulseTag = "<HV_InitialPulse_V1>"
HVTransMultMSMTag = "<HV_TransMultMSM_V1>"
HVRedWHenCircumTag = "<HV_RedWHenCircum_V1>"
HVInfectMultiplierOnARTTag = "<HV_InfectMultiplierOnART_V1>"
HVBloodInfectionTag = "<HV_BloodInfection_V1>"
HVARTReceiveTag = "<HV_ARTReceive_V1>"
HVMonthsInPrimaryStageTag = "<HV_MonthsInPrimaryStage_V1>"
HVGoalsBaseYearIdxTag = "<HV_GoalsBaseYearIdx_V1>"
HVGoalsTargetYearIdxTag = "<HV_GoalsTargetYearIdx_V1>"
HVImpactMatrixTag = "<HV_ImpactMatrix_V1>"
HVImpactMatrixCBIdxTag = "<HV_ImpactMatrixCBIdx_V1>"
HVARTInputCoverageByRGTag = "<HV_ARTInputCoverageByRG_V1>"
HVNumMSMRiskGroupsTag = "<HV_NumMSMRiskGroups_V1>"
HVBalanceSexActsTag = "<HV_BalanceSexActs_V1>"
HVCondomUseRadioGroupIdxTag = "<HV_CondomUseRadioGroupIdx_V1>"
HVCondomUseLogCurveParValTag = "<HV_CondomUseLogCurveParVal_V1>"
HVCondomUseInterpolatedValTag = "<HV_CondomUseInterpolatedVal_V1>"
HVCondomUseLogisticsValTag = "<HV_CondomUseLogisticsVal_V1>"
HVSTIPrevRadioGroupIdxTag = "<HV_STIPrevRadioGroupIdx_V1>"
HVSTIPrevLogCurveParValTag = "<HV_STIPrevLogCurveParVal_V1>"
HVSTIPrevInterpolatedValTag = "<HV_STIPrevInterpolatedVal_V1>"
HVSTIPrevLogisticsValTag = "<HV_STIPrevLogisticsVal_V1>"
HVRiskGroupNamesTag = "<HV_RiskGroupNames_V1>"
HVEdHVSourceTag = "<HV_EdHVSource_V1>"
HVCalcStateDataTag = "<HV_CalcStateData_V1>"
HVAdultsTag = "<HV_Adults_V1>"
HVRiskGroupPercentTag = "<HV_RiskGroupPercent_V1>"
HVNonAIDSDeathRateTag = "<HV_NonAIDSDeathRate_V1>"
HVRateofAgingTag = "<HV_RateofAging_V1>"
HVPopulationsTag = "<HV_Populations_V1>"
HVVaccinatedTag = "<HV_Vaccinated_V1>"
HVTotalVaccinatedTag = "<HV_TotalVaccinated_V1>"
HVUnvaccinatedTag = "<HV_Unvaccinated_V1>"
HVNewVaccinationsTag = "<HV_NewVaccinations_V1>"
HVTotalNewVaccinationsTag = "<HV_TotalNewVaccinations_V1>"
HVTotalAdultsHIVTag = "<HV_TotalAdultsHIV_V1>"
HVNewlyOnARTTag = "<HV_NewlyOnART_V1>"
HVNewlyEligibleARTTag = "<HV_NewlyEligibleART_V1>"
HVTotalAdultsARTTag = "<HV_TotalAdultsART_V1>"
HVRMultAllTag = "<HV_RMultAll_V1>"
HVCalcPrevalenceTag = "<HV_CalcPrevalence_V1>"
HVNewInfectionsTag = "<HV_NewInfections_V1>"
HVExitRateTag = "<HV_ExitRate_V1>"
HVAIDSDeathsTag = "<HV_AIDSDeaths_V1>"
HVAIDSDeathsARTTag = "<HV_AIDSDeathsART_V1>"
HVTotalAIDSDeathsTag = "<HV_TotalAIDSDeaths_V1>"
HVTotalNewInfectionTag = "<HV_TotalNewInfection_V1>"
HVIncidenceTag = "<HV_Incidence_V1>"
HVIncSexRatioTag = "<HV_IncSexRatio_V1>"
HVPercentPopTag = "<HV_PercentPop_V1>"
HVInfectionsAvertedTag = "<HV_InfectionsAverted_V1>"
HVCumInfectionsAvertedTag = "<HV_CumInfectionsAverted_V1>"
HVDeathsAvertedTag = "<HV_DeathsAverted_V1>"
HVCumDeathsAvertedTag = "<HV_CumDeathsAverted_V1>"
HVARTCoverageByRGTag = "<HV_ARTCoverageByRG_V1>"
HVTotalARTCoverageTag = "<HV_TotalARTCoverage_V1>"
HVNewHIVTag = "<HV_NewHIV_V1>"
HVFitDataTag = "<HV_FitData_V1>"
HVFitParamSetTag = "<HV_FitParamSet_V1>"
HVFitControlTag = "<HV_FitControl_V1>"
HVLowRiskCondomUseFromFPTag = "<HV_LowRiskCondomUseFromFP_V1>"
HVEdFmHighlightTag = "<HV_EdFmHighlight_V1>"



import numpy as np
from scipy import interpolate

def goveqs_basis2(input, t,  r, M, i, prm, sel, agg): # % <--- Arguments changed to accommodate prm
    out = np.zeros(i['nx'])#len(input)
    invec = input[0:i['nstates']]

    if t<1980 :
        rHIV=0
    else:
        tempfn = interpolate.interp1d(np.array(range(len(prm['rHIV']))), prm['rHIV'], fill_value="extrapolate")
        ti = t-1980
        rHIV = tempfn(ti) 

    #% r.progression = r.progression.*repelem(((r.betared).^max((t-2003),0)),5)';   % for 5 age groups
    #fac = pow(r['betared'],max(t-2000, 0))#fac = pow(r['betared'][0],max(t-2000, 0))#fac = pow(r['betared'][0],max(t-2003, 0)) #fac = (r.betared)^max(t-2003, 0); % trend on activation and progression rate
    fac_t  = t #fac_t  = min(t,2022); #Carel changed this on 2024-10-18 
    beta_red = r['betared']
    if ((fac_t>=2030) & (fac_t<=2050)):
       #beta_rem = np.linspace(beta_red,1,70)
       #tempfnbta = interpolate.interp1d(np.array(range(len(beta_rem))), beta_rem, fill_value="extrapolate")#beta_red=interp1([2030:2100],[beta_rem(1),beta_rem],fac_t);
       #beta_red = tempfnbta(fac_t-1980)
       beta_rem = np.linspace(beta_red,min(1,beta_red),50)
       tempfnbta = interpolate.interp1d(np.array(range(len(beta_rem))), beta_rem, fill_value="extrapolate")#beta_red=interp1([2030:2100],[beta_rem(1),beta_rem],fac_t);
       beta_red = tempfnbta(fac_t-2030)

    ##fac    = pow(r['betared'],max(fac_t-2000, 0)) #fac    = (r.betared)^max(fac_t-2000, 0); % trend on activation and progression rate
    fac    = pow(beta_red,max(fac_t-2000, 0))

    try :
      #% Normalise by populations
      lam = M['lambda']*invec/sum(invec) #lam = M.lambda*invec/sum(invec);
      nlin_ds = (lam[0]*M['nlin']['a1']['ds']+ lam[1]*M['nlin']['a2']['ds']+ lam[2]*M['nlin']['a3']['ds']+lam[3]*M['nlin']['a4']['ds']+ lam[4]*M['nlin']['a5']['ds'])#nlin_ds = (lam(1)*M.nlin.a1.ds+ lam(2)*M.nlin.a2.ds+ lam(3)*M.nlin.a3.ds+lam(4)*M.nlin.a4.ds+ lam(5)*M.nlin.a5.ds)
      nlin_mdr = lam[5]*M['nlin']['a1']['mdr']+ lam[6]*M['nlin']['a2']['mdr']+ lam[7]*M['nlin']['a3']['mdr'] +lam[8]*M['nlin']['a4']['mdr']+ lam[9]*M['nlin']['a5']['mdr'] #nlin_mdr = lam(6)*M.nlin.a1.mdr+ lam(7)*M.nlin.a2.mdr+ lam(8)*M.nlin.a3.mdr +lam(9)*M.nlin.a4.mdr+ lam(10)*M.nlin.a5.mdr;
      allmat = M['lin'] + M['Dxlin'] + rHIV*M['linHIV'] + fac*M['linLTBI'] + nlin_ds + nlin_mdr # allmat = M.lin + M.Dxlin + rHIV*M.linHIV + fac*M.linLTBI + nlin_ds + nlin_mdr 
      allmat = allmat.toarray()#out = allmat*invec;
      out1 = allmat@invec#np.matmul(allmat, invec_T)
    except:
       KeyboardInterrupt 
            

    # Implement deaths
    mortvec_sum = np.sum(M['mortvec'], 1) #morts = sum(M.mortvec,2).*invec;
    morts = mortvec_sum*invec#morts = np.asmatrix(mortvec_sum*invec).T#[ mortvec_sum[idx]*inv  for idx, inv in enumerate(invec)]
    out1 = out1 - morts

    out[0:i['nstates']] = out1

    # Implement births
    births = sum(morts) + prm['growth'] #births = sum(morts) + prm.growth; #births = sum(morts)
    out[i['U']['a1']['h0']['v0']] = out[i['U']['a1']['h0']['v0']]+births #out(i.U.a1.h0.v0) = out(i.U.a1.h0.v0)+births#out[i['U']['h0']['v0']] = out[i['U']['h0']['v0']]+births

    # Get the auxiliaries
    #out = [o.sum() for o in out]
    out[i['aux']['inc']]  = agg['inc']@(sel['inc']*allmat)@invec #out(i.aux.inc)     = agg.inc*(sel.inc.*allmat)*invec;
    out[i['aux']['noti']] = agg['noti']@(sel['noti']*allmat)@invec #out(i.aux.noti)    = agg.noti*(sel.noti.*allmat)*invec;
    out[i['aux']['noti2']] = sum((sel['noti2']*allmat)@invec) #out(i.aux.noti2)   = sum((sel.noti2.*allmat)*invec);
    out[i['aux']['mort'][0]] = sum(M['mortvec'][:,1]*invec) #out(i.aux.mort(1)) = sum(M.mortvec(:,2).*invec);
    out[i['aux']['mort'][1]] = sum(M['mortvec'][:,2]*invec) #out(i.aux.mort(2)) = sum(M.mortvec(:,3).*invec);

    return out




from SpectrumCommon.Const.GB import *

class SpectrumException(Exception):
    def __init__(self, SWErrorCode, status_code=None, message=None, extra_info={}):
        #self.detail = args[0]
        self.SWErrorCode = SWErrorCode
        self.message = message if message!= None else GB_SW_ExceptMessages[SWErrorCode]
        # self.detail = {'msg':message, 'extra_info':extra_info}
        self.status_code = status_code if status_code!= None else GB_SW_StatusCode[SWErrorCode]
        self.extra_info = extra_info


from SpectrumCommon.Const.IV.IVConst import *
from SpectrumCommon.Const.IV.IVDatabaseKeys import *

from SpectrumCommon.Const.IC.ICTags import *
from SpectrumCommon.Const.IC.ICConst import *
from SpectrumCommon.Const.IC.ICDatabaseConst import *

from SpectrumCommon.Const.HW.HWConst import *
from SpectrumCommon.Const.HW.HWTags import *

########################    IV    ##########################

IH_IV_GROUP_DB_CURR_VERSION        = IV_IH_GROUP_DB_CURR_VERSION
IH_IV_TI_GROUP_DB_CURR_VERSION     = IV_TI_GROUP_DB_CURR_VERSION
IH_IV_CS_GROUP_DB_CURR_VERSION     = IV_CS_GROUP_DB_CURR_VERSION

IH_IV_GROUP_DB_NAME        = IV_GROUP_DB_NAME

# Group DB (GDB)

IH_IV_ROOT_CONSTANT_KEY_GDB    = IV_ROOT_CONSTANT_KEY_GDB
IH_IV_GROUP_MST_ID_KEY_GDB     = IV_GROUP_MST_ID_KEY_GDB
IH_IV_SUBGROUP_MST_ID_KEY_GDB  = IV_SUBGROUP_MST_ID_KEY_GDB   
IH_IV_ENGLISH_STRING_KEY_GDB   = IV_ENGLISH_STRING_KEY_GDB
IH_IV_STRING_CONSTANT_KEY_GDB  = IV_STRING_CONSTANT_KEY_GDB     

#############################    IC    ##########################

IH_IC_DRUG_SUPPLY_DB_NAME         = IC_DRUG_SUPPLY_DB_NAME
IH_IC_DRUG_SUPPLY_DB_CURR_VERSION = IC_DRUG_SUPPLY_DB_CURR_VERSION

IH_IC_MST_ID_KEY_DSDB              = IC_MST_ID_KEY_DSDB    
IH_IC_CONSTANT_KEY_DSDB            = IC_CONSTANT_KEY_DSDB 
IH_IC_ENGLISH_STRING_KEY_DSDB      = IC_ENGLISH_STRING_KEY_DSDB  
IH_IC_COST_TYPE_MST_ID_KEY_LG_DSCB = IC_COST_TYPE_MST_ID_KEY_LG_DSCB  

IH_IC_RES_STATIC_STAFF_TIME_REQ_BY_PROG_AREA_TAG_V1 = IC_RES_STATIC_STAFF_TIME_REQ_BY_PROG_AREA_TAG_V1
IH_IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_PROG_AREA_TAG_V1 = IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_PROG_AREA_TAG_V1

#######################################################################################################
#                                                                                                     #
#                                      HW                                                             #
#                                                                                                     #
#######################################################################################################

IH_HW_ADD_TO_TREAT_INPUTS_MST_ID = HW_ADD_TO_TREAT_INPUTS_MST_ID
IH_HW_NUM_STATIC_STAFF_TYPES     = HW_NUM_STATIC_STAFF_TYPES
IH_HW_NUM_STAFFING_LEVELS        = HW_NUM_STAFFING_LEVELS

IH_HW_STAFF_TYPE_RECORDS_TAG_V1  = HW_STAFF_TYPE_RECORDS_TAG_V1
IH_HW_RES_TOTAL_STAFF_TAG_V1     = HW_RES_TOTAL_STAFF_TAG_V1




from os import environ
from SpectrumCommon.Const.GB.GBConst import GB_SPECT_MOD_DATA_CONN_ENV
from SpectrumCommon.Const.PJ.PJNTags import PJN_ISO3AlphaTag
from SpectrumCommon.Const.DP import AMPreventionsNeedsVersion
from SpectrumCommon.Const.AM import (
    AM_PreventionNeedsFirstTimeTag,
    AM_PreventionNeedsShowVMMCTabTag,
    AM_PreventionNeedsCondomsTag,
    AM_PreventionNeedsVMMCTag,
    AM_PreventionNeedsKeyPopsTag,
    AM_PreventionNeedsPrEPTag
)
from SpectrumCommon.ServerUtils.FilterUtils import get_pjn_modvars, set_all_modvars
from AvenirCommon.Database import GB_get_db_json

def ReadPreventionNeeds_KeyPopulations(PreventionNeedsDB):
    return PreventionNeedsDB.get('KeyPopulations', {})

def ReadPreventionNeeds_VMMC(PreventionNeedsDB):
    return PreventionNeedsDB.get('VMMC', [])

def ReadPreventionNeeds_Condoms(PreventionNeedsDB):
    return PreventionNeedsDB.get('Condoms', {})

def ReadPreventionNeeds_PrEP(PreventionNeedsDB):
    return PreventionNeedsDB.get('PrEP', {})

def reload_prevention_needs(projID, modvarTags=None):
    pjn_modvars = get_pjn_modvars(projID)
    ISO3 = pjn_modvars[PJN_ISO3AlphaTag]
    PreventionNeedsDB = GB_get_db_json(environ[GB_SPECT_MOD_DATA_CONN_ENV], 
                                        'aim', 
                                        f'PreventionNeedsDB/{ISO3}_{AMPreventionsNeedsVersion}.JSON')
    
    aim_modvars = {
        AM_PreventionNeedsFirstTimeTag   : True,
        AM_PreventionNeedsShowVMMCTabTag : 'VMMC'in PreventionNeedsDB,
        AM_PreventionNeedsCondomsTag     : ReadPreventionNeeds_Condoms(PreventionNeedsDB),
        AM_PreventionNeedsVMMCTag        : ReadPreventionNeeds_VMMC(PreventionNeedsDB),
        AM_PreventionNeedsKeyPopsTag     : ReadPreventionNeeds_KeyPopulations(PreventionNeedsDB),
        AM_PreventionNeedsPrEPTag        : ReadPreventionNeeds_PrEP(PreventionNeedsDB)
    }
    if modvarTags and len(modvarTags) > 0:
        aim_modvars = {k: v for k, v in aim_modvars.items() if k in modvarTags}
    set_all_modvars(projID, aim_modvars)
    return aim_modvars# "Reloaded Prevention Needs Successfully"


from SpectrumCommon.Modvars.DP.DPModvarShapes import DP_modvar_shapes
from SpectrumCommon.Modvars.AM.AMModvarShapes import AM_modvar_shapes
from SpectrumCommon.Modvars.FP.FPModvarShapes import FP_modvar_shapes
from SpectrumCommon.Modvars.TB.TBModvarShapes import TB_modvar_shapes
from SpectrumCommon.Modvars.IH.IHModvarShapes import IH_modvar_shapes

spectrum_modvar_shapes = {
    'demproj':DP_modvar_shapes,
    'aim':AM_modvar_shapes,
    'famplan':FP_modvar_shapes,
    'tuberculosis':TB_modvar_shapes,
    'iht':IH_modvar_shapes
}


import time
import json
import os
from .SpectrumWebConnect import run_dynamical_model


#retrieve modvars from json file
final_model_path = os.getcwd()+'\\FinalModel2'
#with open(f"{final_model_path}\\data\\CtriesDefaults\\NGA_modvars.JSON", "r") as fp:
#    modvars = json.load(fp)


with open(f"{final_model_path}\\data\\CtriesDefaults\\TB_NGA_10_21.JSON", "r") as fp:
    modvars = json.load(fp)

#Copied constants from spectrum engine
TB_DynPRMTag           = '<TB_DynamicalModelPRM_V1>'
TB_DynPopDataTag       = '<TB_DynamicalModelPopData_V1>'
TB_CounterfactualScenarioTag  = '<TB_CounterfactualScenario_V1>'
TB_CurrentScenarioTag         = '<TB_CurrentScenario_V1>'
# Scenarios 
TB_AvgProbDetect      = 0 
TB_AvgProbReceivPrvnt = 1
TB_TreatmentSuccess   = 2
TB_ScenarioLen = 3

#run the dynamical model
start_time = time.time()
results = run_dynamical_model('NGA',
                            modvars[TB_DynPRMTag],
                            None, # modvars[tbc.TB_DynXSTag],
                            modvars[TB_DynPopDataTag],
                            modvars[TB_CurrentScenarioTag][TB_AvgProbReceivPrvnt][-1],
                            modvars[TB_CounterfactualScenarioTag][TB_AvgProbReceivPrvnt][-1],
                            modvars[TB_CurrentScenarioTag][TB_TreatmentSuccess][-1],
                            modvars[TB_CounterfactualScenarioTag][TB_TreatmentSuccess][-1],
                            modvars[TB_CurrentScenarioTag][TB_AvgProbDetect][-1],
                            modvars[TB_CounterfactualScenarioTag][TB_AvgProbDetect][-1],
                            final_model_path)
end_time = time.time()
print(f'time {end_time-start_time}')



import numpy as np
from os import environ
from copy import deepcopy

from AvenirCommon.Database import GB_get_db_json, GB_file_exists
from SpectrumCommon.Const.GB import GB_BaseYear 
from SpectrumCommon.Const.FP import *
from SpectrumCommon.Modvars.FP.FPModvarDef import *


from SpectrumCommon.Const.DP import *


#@autoreload
def init_modvars(countryID, first_year, final_year,  extra):
    #Variables needed for multiple modvars
    #final_year = 2030
    # if countryID in FP_COUNTRY_PROXY:
    #     countryID = FP_COUNTRY_PROXY[countryID]
    
    #base_year = GB_BaseYear
    #first_year_idx = first_year - base_year
    final_year_idx = final_year - first_year 

    # db_first_yr_idx = first_year-DP_DefaultSurveyYear
    # db_final_yr_idx = final_year-DP_DefaultSurveyYear
    db_first_yr_idx = first_year-1990
    db_final_yr_idx = final_year-1990
    
    exploreLiST = extra['exploreLiST']
    RapidProj = extra['RapidProj']
    # LiSTProj = extra['LiSTProj']
    
    mode = FP_MARUninterpolated if exploreLiST else FP_MAR

    if RapidProj:
        mode = FP_AllUninterpolated if exploreLiST else FP_All

    # if LiSTProj:
    #     mode = FP_MARUninterpolated if exploreLiST else FP_MAR
    
    dir = FP_DB_Dir[mode] + '/' + FP_DB_SubDir[mode]
    version = FP_DB_Version[mode]
    
    #Load databases
    db = GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'famplan', f'{dir}/{countryID}_{version}.JSON') 
    eff_db = GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'famplan', 'FPEffectiveness_V1.JSON') 
    
    #extend database years
    for key in db:
        if type(db[key]) is list:
            for age in range(FP_All_Ages, FP_MAX_AGE+1) :
                if final_year>FP_FinalDBYear:
                    #flatline years   
                    for yr in range(FP_FinalDBYear, final_year):  
                        last_item = db[key][age][-1]
                        db[key][age].append(last_item)  
                elif final_year<FP_FinalDBYear:
                    #truncate years
                    for yr in range(final_year, FP_FinalDBYear):  
                       db[key][age].pop(-1)
                    
                if first_year<FP_FirstDBYear:
                    #flatline years   
                    for yr in range(first_year, FP_FirstDBYear):  
                        first_item = db[key][age][0]
                        db[key][age].insert(0, first_item)
                        pass
                
                
                #cut off historical years, leaving values only between first and final year
                if db_first_yr_idx>0:
                    db[key][age] = db[key][age][db_first_yr_idx:db_final_yr_idx+1]
            db[key] = np.array(db[key])

    

    #replace baseline tfr with tfr from demproj file
    DP_TFR = FP_GetDPTFR(countryID, first_year, final_year)
    if DP_TFR:
        db['TFR'][0] = DP_TFR
    year_zeroes = np.zeros((final_year_idx+1))
    age_year_zeroes = np.zeros((FP_MAX_AGE+1, final_year_idx+1))
    age_zeroes = np.zeros((FP_MAX_AGE+1))
    method_src_year_zeroes = np.zeros((FP_Max_Methods+1, FP_max_sources+1, final_year_idx+1))
    age_need_year_zeroes =  np.zeros((FP_MAX_AGE+1, FP_max_need+1, final_year_idx+1))
    
    method_age_src_year_zeroes = np.zeros((FP_Max_Methods+1, FP_MAX_AGE+1, FP_max_sources+1, final_year_idx+1))

    method_src_year_hundred = np.full((FP_Max_Methods+1, FP_max_sources+1, final_year_idx+1), 100)

    # abortion_percent = np.full((FP_MAX_AGE+1, final_year_idx+1), 32)
    abortion_percent = np.zeros((FP_MAX_AGE+1, final_year_idx+1))
    # abortion_percent[FP_All_Ages] += db['abortionsPerUnwantedPregnancy']
    abortion_percent[FP_All_Ages] += db['PropUnintendedPregEndInAbort']*100

    sterility =  db['childlessWomen45to49Sterility']

    in_union = db['inUnion']
    WRA = Fp_GetDPWRAFlatlined (countryID, first_year, final_year_idx, extra)
    MWRA = FP_ResetMWRA(final_year_idx, in_union, WRA, [FP_All_Ages])

    method_mix = MethodMix_Init(db, final_year_idx)
    effectiveness = Effectiveness_Init(eff_db, final_year_idx)
    avg_effectiveness =  FP_AverageEffectiveness(final_year_idx, method_mix, effectiveness)

    effectiveness_meta = deepcopy(effectiveness)

    #modvar creation list
    return {
        # db,
        #General
        FP_ActiveMethodsTag                : ActiveMethod_Init(db),
        FP_AgeGroupOptionTag               : FP_Single_Age_Group,
        FP_GoalTag                         : FP_Prevalence,
        FP_AbortionOptionTag               : FP_calculate_abortions,
        FP_NeedOptionTag                   : False,
        FP_SourcesTag                      : Sources_Init(),
        FP_MethodNameTag                   : MethodName_Init(),
        FP_NumSourcesTag                   : 1,
        FP_NumNeedsTag                     : FP_max_need,
        FP_NumCustomMethodsTag             : 0,
        FP_PregnancyRateTag                : 0.42,
        FP_PercentPregWithMiscarriageTag   : 0.13,
        FP_PropUnintendedPregEndInAbortTag : db['PropUnintendedPregEndInAbort'],
        FP_PropUnsafeAbortTag              : db['propUnsafeAbort'],
        FP_MortalityRateSafeAbortTag       : 2,
        FP_MortalityRateUnsafeAbortTag     : db['MortalityRateUnsafeAbort'],
        FP_PropPregWantedLaterTag          : 0,
        FP_PropPregUnwantedTag             : 0,
        FP_MiscarriagePercentTag           : FP_MiscarriagePercent,
        FP_CalcMethOptionTag               : FP_TotFec,
        FP_MethodMixTag                    : method_mix,
        FP_MethodPermTermTag               : FP_PermanentMethods,
        FP_MethodLongTermTag               : FP_LongTermtMethods,
        FP_MethodshortTermTag              : FP_ShortTermMethods,
        FP_EffectivenessTag                : effectiveness,
        FP_EffectivenessMetaTag            : effectiveness_meta, #Is this needed? Yes, it is needed for RAPID
        FP_UnitsPerCYPTag                  : UnitsPerCYP_Init(final_year_idx),
        FP_AverageAgeTag                   : AverageAge_Init(countryID, final_year_idx),
        FP_DurationOfUseTag                : DurationOfUse_Init(final_year_idx),
        FP_SourceMixTag                    : method_src_year_hundred,
        FP_CostPerUATag                    : method_src_year_zeroes,
        FP_FeesTag                         : method_src_year_zeroes,
        FP_GrossCostTag                    : method_src_year_zeroes,
        FP_RevenueTag                      : method_src_year_zeroes,
        FP_NetCostTag                      : method_src_year_zeroes,
        FP_CommoditiesTag                  : method_src_year_zeroes,
        FP_PercentInUnionTag               : in_union,
        FP_PPITag                          : db['PPI_MedianDuration'],
        FP_TARTag                          : db['TAR'],
        FP_AbortionPercentTag              : abortion_percent,
        FP_SterilityTag                    : sterility,
        FP_MarriageIndexTag                : age_year_zeroes, 
        FP_PPIIndexTag                     : age_year_zeroes, 
        FP_AbortionIndexTag                : age_year_zeroes,
        FP_SterilityIndexTag               : age_year_zeroes,
        FP_PrevalenceIndexTag              : age_year_zeroes,
        FP_TFRTag                          : db['TFR'],
        FP_TFRMetaTag                      : {'default': db['TFR'].tolist()}, #is this needed? Yes, it is needed for RAPID
        FP_WRATag                          : WRA,
        FP_MWRATag                         : MWRA,
        FP_MortalityRateTag                : age_year_zeroes,
        FP_BirthsTag                       : age_year_zeroes,
        FP_BirthsAvertedTag                : age_year_zeroes,
        FP_AbortionsTag                    : age_year_zeroes,
        FP_AverageEffectivenessTag         : avg_effectiveness,
        FP_PrevalenceTag                   : prevalence_Init(db['CPR'], final_year_idx), 
        FP_MPrevalenceTag                  : age_need_year_zeroes,
        FP_AddMdrnUsersPrevTag             : age_need_year_zeroes,
        FP_UnmetNeedTag                    : unmetNeed_Init(db['unmetNeed'], final_year_idx),
        FP_UnmetNeedReductionTag           : np.zeros((FP_max_need+1, final_year_idx+1)),
        FP_WantedTFRTag                    : age_zeroes,
        FP_TFRdifferenceTag                : year_zeroes,
        FP_TestUnmetNeedFromPrevTag        : True,
        FP_CalcCostRegressionTag           : False,
        FP_ASFRInput_TFRGoalTag            : FP_TFR_MT_TFRGOAL,
        FP_ChildSurvInputsTag              : ChildSurvInputs_Init(db),
        FP_PACInputsTag                    : PACInputs_Init(db['MMR'], final_year_idx),
        FP_LegalAbortNeedTreatTag          : 0,
        FP_PACOutputsTag                   : np.zeros((FP_CostPerMaternDeathAvert+1, final_year_idx+1)),  
        FP_LAMinsusceptibleTag             : LAMinsusceptible_Init(),
        FP_EPSurveyYearTFRTag              : 0,
        FP_TFTag                           : db['TF'],
        FP_PRTag                           : age_zeroes,
        FP_DiscontinuationRateTag          : np.zeros((FP_Max_Methods+1, final_year_idx+1)),  
        FP_UsersTag                        : method_age_src_year_zeroes,
        FP_AcceptorsTag                    : method_age_src_year_zeroes,
        FP_GrowthRateTag                   : method_age_src_year_zeroes,
        FP_PregnanciesTag                  : np.zeros((FP_MAX_AGE+1, FP_mistimed+1, final_year_idx+1)),  
        FP_ChildSurvOutputsTag             : np.zeros((FP_FertAdjU5MR+1, final_year_idx+1)), 
        FP_PregAvertedTag                  : year_zeroes,
        FP_MaternalDeathsAvertedTag        : year_zeroes,
        FP_UnsafeAbortionsAvertedTag       : year_zeroes,
        FP_RelativeRisksTag                : relativeRisks_Init(db, final_year_idx),
        FP_SourceTag                       : FPSource_Init(db),
        FP_DataSourceTag                   : db['mainDataSources'],
        FP_AvgHouseholdTag                 : 0,#db['householdSize'],
        FP_GoalAnnualChangeTag             : [{'useAnnualChange' :False, 'annualChange' :0} for g in GBRange(0, FP_Pregnancy)],
        FP_RRAgeBirthOrderResultTag        : np.zeros((FP_RRBirthOrderMax + 1, FP_RRAgeOrderMax + 1, final_year_idx+1)),
        FP_RRBirthIntervalResultTag        : np.zeros((FP_RRBirthIntervalMax + 1, final_year_idx+1))
    }




# Get WRA from DP and set in FP *)
def Fp_GetDPWRAFlatlined (countryID, firstYear, final_year_idx, extra):
    subnatCode = GBGetKeyValue(extra, 'subNatCode', 0)
    # UPDCountryName = formatCountryFName(countryID, DPUPDVersion, subnatCode)\
    # if GB_file_exists(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'demproj', 'UPD/' + UPDCountryName):
        # UPDdb = GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'demproj', 'UPD/' + UPDCountryName) 
    pop1CountryName = formatCountryFName(countryID, DPInitialConditionsDBVersion)#, subnatCode)
    if GB_file_exists(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'demproj', DPInitialConditionsDBSubDir + pop1CountryName):
        Pop1DefaultDB = np.array(GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'demproj', DPInitialConditionsDBSubDir + pop1CountryName)['pop1'])
        
        first_year_pop = Pop1DefaultDB[firstYear - GB_BaseYear, :, :, :, :]
        female_pop = first_year_pop[1, :, :, :]

        WRA = np.zeros((FP_MAX_AGE+1, final_year_idx+1))
            # +3 is because FP at age 15-20 starts at the 4 id for DP, offset +3*)
        WRA[FP_A15_19] += np.sum(female_pop[15:20])
        WRA[FP_A20_24] += np.sum(female_pop[20:25])
        WRA[FP_A25_29] += np.sum(female_pop[25:30])
        WRA[FP_A30_34] += np.sum(female_pop[30:35])
        WRA[FP_A35_39] += np.sum(female_pop[35:40])
        WRA[FP_A40_44] += np.sum(female_pop[40:45])
        WRA[FP_A45_49] += np.sum(female_pop[45:50])
        
        WRA[0] = np.sum(WRA[1:], axis=0)

        return WRA
    
def FP_GetDPTFR(countryID, first_year, final_year):
    
    connection = environ[GB_SPECT_MOD_DATA_CONN_ENV]
    UPDCountryName = formatCountryFName(countryID, DPUPDVersion)#, subnatCode)
    if GB_file_exists(connection, 'demproj', 'UPD/' + UPDCountryName):
        UPDdb = GB_get_db_json(connection, 'demproj', 'UPD/' + UPDCountryName) 
    else:
        return None
    
    dataFirstYear = 1970
    dataFinalYear = DP_UPDFinalYear
#  
    DP_TFR = []

    for year in GBRange(first_year, min(final_year, dataFinalYear)):
        DP_TFR.append(UPDdb['tfr'][str(year)])

    for year in GBRange(dataFinalYear + 1, final_year):
        DP_TFR.append(UPDdb['tfr'][str(dataFinalYear)])
    
    return DP_TFR



def FP_ResetMWRA(final_year_idx, in_union, WRA, ages):
    
    MWRA = np.zeros((FP_MAX_AGE+1, final_year_idx+1))
    for a in ages:
        MWRA[a] = WRA[a]*in_union[a]/100
    
    if len(ages)>1:
        MWRA[FP_All_Ages] = np.sum(MWRA[1:], axis=0)

    return MWRA
            

# Calculate average effectiveness
def FP_AverageEffectiveness(final_year_idx, method_mix, effectiveness):
    avg_effectiveness = np.zeros((FP_MAX_AGE+1, FP_max_need+1, final_year_idx+1))
    
    mix_effect = np.sum(method_mix[:,:,FP_all_need,:]*effectiveness, axis=1)/np.sum(method_mix[:,:,FP_all_need,:], axis=1)

    avg_effectiveness[:, FP_all_need, :] = mix_effect
    return avg_effectiveness



    # FP := GetFPData(proj);
    # for a := FP_All_Ages to FP_MAX_AGE do
    # begin
    #     mixSum := 0;
    #     mixEffect := 0;
    #     for m := 1 to FP_Max_Methods do
    #     begin
    #     if FP.GetmethodActive(m) then //and (m <> FP_LAM)
    #     begin
    #         mixSum := mixSum + FP.GetMethodMix(a,m,FP_All_Need,t);
    #         mixEffect := mixEffect + FP.GetMethodMix(a,m,FP_All_Need,t)
    #                                 *FP.GetEffectiveness(a,m,t);
    #     end;
    #     end;
    #     if mixSum<>0 then
    #     FP.SetAverageEffectiveness(a,FP_All_Need, t, mixEffect/mixSum);



import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.IH.IHModVarFields as ihmvf

#######################################################################################################
#                                                                                                     #
#                                       Programme areas                                               #
#                                                                                                     #
#######################################################################################################

def get_IH_num_prog_areas(prog_area_records: list):
    return len(prog_area_records)

def get_IH_prog_area_rec(prog_area_records: list, ID: str):
    prog_area_recs = [prog_area_rec for prog_area_rec in prog_area_records if prog_area_rec[ihmvf.IH_PROG_AREA_ID] == ID]
    return prog_area_recs[0] if len(prog_area_recs) > 0 else gbc.GB_NOT_FOUND

    # prog_area_dict = None
    
    # pa = 0
    # stop = False
    # num_prog_areas = get_IH_num_prog_areas(prog_area_dict_list)

    # while (pa < num_prog_areas) and (not stop):

    #     prog_area_ID = get_IH_prog_area_ID(prog_area_dict_list[pa])

    #     if prog_area_ID == ID:
    #         prog_area_dict = prog_area_dict_list[pa]
    #         stop = True

    #     pa += 1

    # return prog_area_dict

def get_IH_prog_area_idx(prog_area_records: list, ID: str):
    indices = [idx for (idx, prog_area_rec) in enumerate(prog_area_records) if prog_area_rec[ihmvf.IH_PROG_AREA_ID] == ID]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND

    # idx = gbc.GB_NOT_FOUND
    
    # pa = 0
    # stop = False
    # num_prog_areas = get_IH_num_prog_areas(prog_area_dict_list)

    # while (pa < num_prog_areas) and (not stop):

    #     prog_area_ID = get_IH_prog_area_ID(prog_area_dict_list[pa])

    #     if prog_area_ID == ID:
    #         idx = pa
    #         stop = True

    #     pa += 1

    # return idx

###

def get_IH_prog_area_name(prog_area_rec: dict):
    return prog_area_rec[ihmvf.IH_PROG_AREA_NAME]

def set_IH_prog_area_name(prog_area_rec: dict, value: str):
    prog_area_rec[ihmvf.IH_PROG_AREA_NAME] = value

###

def get_IH_prog_area_string_constant(prog_area_rec: dict):
    return prog_area_rec[ihmvf.IH_PROG_AREA_STR_CONSTANT]

def set_IH_prog_area_string_constant(prog_area_rec: dict, value: str):
    prog_area_rec[ihmvf.IH_PROG_AREA_STR_CONSTANT] = value

###

def get_IH_prog_area_ID(prog_area_rec: dict):
    return prog_area_rec[ihmvf.IH_PROG_AREA_ID]

def set_IH_prog_area_ID(prog_area_rec: dict, value: str):
    prog_area_rec[ihmvf.IH_PROG_AREA_ID] = value

###

def get_IH_prog_area_active(prog_area_rec: dict):
    return prog_area_rec[ihmvf.IH_PROG_AREA_ACTIVE]

def set_IH_prog_area_active(prog_area_rec: dict, value: bool):
    prog_area_rec[ihmvf.IH_PROG_AREA_ACTIVE] = value

###

def get_IH_prog_area_custom(prog_area_rec: dict):
    return prog_area_rec[ihmvf.IH_PROG_AREA_CUSTOM]

def set_IH_prog_area_custom(prog_area_rec: dict, value: bool):
    prog_area_rec[ihmvf.IH_PROG_AREA_CUSTOM] = value

###

def get_IH_prog_area_subgroups(prog_area_dict: dict):
    return prog_area_dict[ihmvf.IH_PROG_AREA_SUBGROUP_RECORDS]

#######################################################################################################
#                                                                                                     #
#                                       Subgroups                                                     #
#                                                                                                     #
#######################################################################################################

def get_IH_num_subgroups(subgroup_records: list):
    return len(subgroup_records)

def get_IH_subgroup_rec(prog_area_records: list, prog_area_ID: str, ID: str):
    prog_area_rec = get_IH_prog_area_rec(prog_area_records, prog_area_ID)
    subgroup_records = get_IH_prog_area_subgroups(prog_area_rec)

    subgroup_recs = [subgroup_rec for subgroup_rec in subgroup_records if subgroup_rec[ihmvf.IH_SUBGROUP_ID] == ID]
    return subgroup_recs[0] if len(subgroup_recs) > 0 else gbc.GB_NOT_FOUND

    # sg = 0
    # stop = False
    # num_subgroups = get_IH_num_subgroups(subgroup_records)

    # while (sg < num_subgroups) and (not stop):

    #     subgroup_ID = get_IH_subgroup_ID(subgroup_records[sg])

    #     if subgroup_ID == ID:
    #         subgroup_rec = subgroup_records[sg]
    #         stop = True

    #     sg += 1

    # return subgroup_rec

def get_IH_subgroup_idx(prog_area_records: list, prog_area_ID: str, ID: str):
    prog_area_rec = get_IH_prog_area_rec(prog_area_records, prog_area_ID)
    subgroup_records = get_IH_prog_area_subgroups(prog_area_rec)
    
    indices = [idx for (idx, subgroup_rec) in enumerate(subgroup_records) if subgroup_rec[ihmvf.IH_SUBGROUP_ID] == ID]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND

    # idx = gbc.GB_NOT_FOUND

    # sg = 0
    # stop = False
    # num_subgroups = get_IH_num_subgroups(subgroup_records)

    # while (sg < num_subgroups) and (not stop):

    #     subgroup_ID = get_IH_subgroup_ID(subgroup_records[sg])

    #     if subgroup_ID == ID:
    #         idx = sg
    #         stop = True

    #     sg += 1

    # return idx

def get_IH_subgroup_name(subgroup_rec: dict):
    return subgroup_rec[ihmvf.IH_SUBGROUP_NAME]

def set_IH_subgroup_name(subgroup_rec: dict, value: str):
    subgroup_rec[ihmvf.IH_SUBGROUP_NAME] = value

###

def get_IH_subgroup_string_constant(subgroup_rec: dict):
    return subgroup_rec[ihmvf.IH_SUBGROUP_STR_CONSTANT]

def set_IH_subgroup_string_constant(subgroup_rec: dict, value: str):
    subgroup_rec[ihmvf.IH_SUBGROUP_STR_CONSTANT] = value

###

def get_IH_subgroup_ID(subgroup_rec: dict):
    return subgroup_rec[ihmvf.IH_SUBGROUP_ID]

def set_IH_subgroup_ID(subgroup_rec: dict, value: str):
    subgroup_rec[ihmvf.IH_SUBGROUP_ID] = value

###

def get_IH_subgroup_active(subgroup_rec: dict):
    return subgroup_rec[ihmvf.IH_SUBGROUP_ACTIVE]

def set_IH_subgroup_active(subgroup_rec: dict, value: bool):
    subgroup_rec[ihmvf.IH_SUBGROUP_ACTIVE] = value

###

def get_IH_subgroup_custom(subgroup_rec: dict):
    return subgroup_rec[ihmvf.IH_SUBGROUP_CUSTOM]

def set_IH_subgroup_custom(subgroup_rec: dict, value: bool):
    subgroup_rec[ihmvf.IH_SUBGROUP_CUSTOM] = value

#######################################################################################################
#                                                                                                     #
#                                      Delivery channels                                              #
#                                                                                                     #
#######################################################################################################

def get_IH_num_deliv_chans(deliv_chan_records: list):
    return len(deliv_chan_records)

def get_IH_deliv_chan_rec_from_curr_ID(deliv_chan_records: list, curr_ID: int):
    return deliv_chan_records[curr_ID - 1]

def get_IH_deliv_chan_rec(deliv_chan_records: list, ID: str):
    deliv_chan_recs = [deliv_chan_rec for deliv_chan_rec in deliv_chan_records if deliv_chan_rec[ihmvf.IH_DELIV_CHAN_ID] == ID]
    return deliv_chan_recs[0] if len(deliv_chan_recs) > 0 else gbc.GB_NOT_FOUND

    # deliv_chan_rec = None
    
    # dc = 0
    # stop = False
    # num_deliv_chans = get_IH_num_deliv_chans(deliv_chan_records)

    # while (dc < num_deliv_chans) and (not stop):

    #     deliv_chan_ID = get_IH_deliv_chan_ID(deliv_chan_records[dc])

    #     if deliv_chan_ID == ID:
    #         deliv_chan_rec = deliv_chan_records[dc]
    #         stop = True

    #     dc += 1

    # return deliv_chan_rec

def get_IH_deliv_chan_idx(deliv_chan_records: list, ID: str):
    indices = [idx for (idx, deliv_chan_rec) in enumerate(deliv_chan_records) if deliv_chan_rec[ihmvf.IH_DELIV_CHAN_ID] == ID]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND

    # idx = gbc.GB_NOT_FOUND
    
    # dc = 0
    # stop = False
    # num_deliv_chans = get_IH_num_deliv_chans(deliv_chan_records)

    # while (dc < num_deliv_chans) and (not stop):

    #     deliv_chan_ID = get_IH_deliv_chan_ID(deliv_chan_records[dc])

    #     if deliv_chan_ID == ID:
    #         idx = dc
    #         stop = True

    #     dc += 1

    # return idx

###

def get_IH_deliv_chan_name(deliv_chan_rec: dict):
    return deliv_chan_rec[ihmvf.IH_DELIV_CHAN_NAME]

def set_IH_deliv_chan_name(deliv_chan_rec: dict, value: str):
    deliv_chan_rec[ihmvf.IH_DELIV_CHAN_NAME] = value

###

def get_IH_deliv_chan_string_constant(deliv_chan_rec: dict):
    return deliv_chan_rec[ihmvf.IH_DELIV_CHAN_STR_CONSTANT]

def set_IH_deliv_chan_string_constant(deliv_chan_rec: dict, value: str):
    deliv_chan_rec[ihmvf.IH_DELIV_CHAN_STR_CONSTANT] = value

###

def get_IH_deliv_chan_ID(deliv_chan_rec: dict):
    return deliv_chan_rec[ihmvf.IH_DELIV_CHAN_ID]

def set_IH_deliv_chan_ID(deliv_chan_rec, value: str):
    deliv_chan_rec[ihmvf.IH_DELIV_CHAN_ID] = value

###

def get_IH_deliv_chan_active(deliv_chan_rec: dict):
    return deliv_chan_rec[ihmvf.IH_DELIV_CHAN_ACTIVE]

def set_IH_deliv_chan_active(deliv_chan_rec: dict, value: bool):
    deliv_chan_rec[ihmvf.IH_DELIV_CHAN_ACTIVE] = value

###

def get_IH_deliv_chan_custom(deliv_chan_rec: dict):
    return deliv_chan_rec[ihmvf.IH_DELIV_CHAN_CUSTOM]

def set_IH_deliv_chan_custom(deliv_chan_rec: dict, value: bool):
    deliv_chan_rec[ihmvf.IH_DELIV_CHAN_CUSTOM] = value

###
    
#######################################################################################################
#                                                                                                     #
#                              Health system administrative units                                     #
#                                                                                                     #
#######################################################################################################

def get_IH_num_health_sys_admin_units(health_sys_admin_unit_records: list):
    return len(health_sys_admin_unit_records)

def get_IH_health_sys_admin_unit_rec(health_sys_admin_unit_records: list, ID: str):
    health_sys_admin_unit_recs = [health_sys_admin_unit_rec for health_sys_admin_unit_rec in health_sys_admin_unit_records if health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_ID] == ID]
    return health_sys_admin_unit_recs[0] if len(health_sys_admin_unit_recs) > 0 else gbc.GB_NOT_FOUND

def get_IH_health_sys_admin_unit_idx(health_sys_admin_unit_records: list, ID: str):
    indices = [idx for (idx, health_sys_admin_unit_rec) in enumerate(health_sys_admin_unit_records) if health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_ID] == ID]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND

###

def get_IH_health_sys_admin_unit_name(health_sys_admin_unit_rec: dict):
    return health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_NAME]

def set_IH_health_sys_admin_unit_name(health_sys_admin_unit_rec: dict, value: str):
    health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_NAME] = value

###

def get_IH_health_sys_admin_unit_string_constant(health_sys_admin_unit_rec: dict):
    return health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_STR_CONSTANT]

def set_IH_health_sys_admin_unit_string_constant(health_sys_admin_unit_rec: dict, value: str):
    health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_STR_CONSTANT] = value

###

def get_IH_health_sys_admin_unit_ID(health_sys_admin_unit_rec: dict):
    return health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_ID]

def set_IH_health_sys_admin_unit_ID(health_sys_admin_unit_rec: dict, value: str):
    health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_ID] = value

###

def get_IH_health_sys_admin_unit_active(health_sys_admin_unit_rec: dict):
    return health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_ACTIVE]

def set_IH_health_sys_admin_unit_active(health_sys_admin_unit_rec: dict, value: bool):
    health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_ACTIVE] = value

###

def get_IH_health_sys_admin_unit_custom(health_sys_admin_unit_rec: dict):
    return health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_CUSTOM]

def set_IH_health_sys_admin_unit_custom(health_sys_admin_unit_rec: dict, value: bool):
    health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_CUSTOM] = value

###

def get_IH_health_sys_admin_unit_number(health_sys_admin_unit_rec: dict):
    return health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_NUMBER]

def set_IH_health_sys_admin_unit_number(health_sys_admin_unit_rec: dict, value = int):
    health_sys_admin_unit_rec[ihmvf.IH_HEALTH_SYS_ADMIN_UNIT_NUMBER] = value

#######################################################################################################
#                                                                                                     #
#                                      Delivery channel packages                                      #
#                                                                                                     #
#######################################################################################################

#######################################################################################################
#                                                                                                     #
#                                      National programmes                                            #
#                                                                                                     #
#######################################################################################################

#######################################################################################################
#                                                                                                     #
#                                      Years                                                          #
#                                                                                                     #
#######################################################################################################